from .html_slot_config import *


class ShouldIndex:
    def __init__(self):
        # 裁剪指标
        self.crop_indexing_dict = {
            'extension': {'status': False},
            'main_part_offset': {'coordinates': 'x', 'offset': 0},
            'crop_head': {'status': False},
            'get_crop_point': {'status': False},
            'slot_and_img_ration': {'offset': 1},
            'expected_crop_result': False,
            # 上下padding 留了多少
            'top_padding_ratio': 0,
            'bottom_padding_ratio': 0,
        }

    def expected_crop_result(self):
        if self.crop_indexing_dict['get_crop_point']['status']:
            self.crop_indexing_dict['expected_crop_result'] = True

    def extension(self):
        self.crop_indexing_dict['extension'] = {
            'status': True
        }

    def main_part_offset(self, pic_center, main_part_center, coordinates):
        """
        图片主体内容 偏移 图片中心的比率
        :param pic_center:  裁剪后中点
        :param main_part_center: 主题中点
        :param coordinates:  x 轴 或 y轴 ,原点在左上方处
        :return:
        """

        if pic_center < main_part_center:
            offset = - (main_part_center - pic_center) / pic_center
        elif pic_center == main_part_center:
            offset = 0
        else:
            offset = (pic_center - main_part_center) / pic_center

        self.crop_indexing_dict['main_part_offset'] = {
            'coordinates': coordinates,
            'offset': offset
        }

        return offset

    def img_scale_ratio(self, width_scale, height_scale):
        """
        图片缩放比率 ,目前图片都是等比放缩。所以 宽高缩放率是一样的
        小于1  为缩放
        大于1  为放大
        :param width_scale:
        :param height_scale:
        :return:
        """
        self.crop_indexing_dict['img_scale_ratio'] = {
            'width_scale': width_scale,
            'height_scale': height_scale,
        }

        return width_scale

    def get_slot_and_img_ration(self, slot_ratio, crop_main_part_ratio):
        """
        slot宽高比 / 裁剪主题宽高比
        :param slot_ratio:
        :param crop_main_part_ratio:
        :return:
        """
        self.crop_indexing_dict['slot_and_img_ration'] = {
            'slot_ratio': slot_ratio,
            'img_ratio': crop_main_part_ratio,
            'offset': abs(crop_main_part_ratio - slot_ratio) / slot_ratio
        }
        return None

    def get_crop_site(self, origin_crop_site, crop_site):
        """预设裁剪点是否取到 或 兼容取到"""

        self.crop_indexing_dict['get_crop_point'] = {
            'origin_crop_site': origin_crop_site,
            'crop_site': crop_site,
            'status': True if origin_crop_site == crop_site else False,
        }

    def get_top_and_bottom_padding(self, top, padding_top, bottom, padding_bottom):
        """获取 上下 padding占比 """
        if top >= padding_top:
            top_padding_ratio = 1
        else:
            top_padding_ratio = top / padding_top
        self.crop_indexing_dict['top_padding_ratio'] = top_padding_ratio

        if bottom + padding_bottom <= 1:
            bottom_padding_ratio = 1
        else:
            bottom_padding_ratio = (1 - bottom) / padding_bottom

        self.crop_indexing_dict['bottom_padding_ratio'] = bottom_padding_ratio


    def crop_head(self):
        """裁剪头"""
        self.crop_indexing_dict['crop_head'] = {
            'status': True
        }


class ImageCrop:

    def __init__(self):
        self.si = ShouldIndex()
        # 裁剪点 向上向下兼容顺序顺序
        self.crop_sequence = ['pic_top', 'human_top', 'eye', 'nose', 'ear', 'mouth', 'mandible', 'shoulder',
                              'elbow',
                              'wrist', "waist", 'hip', 'knee', 'ankle', 'human_bottom', 'pic_bottom']
        self.garment_crop_sequence = ['pic_top', 'human_top', 'garment_top', 'garment_bottom', 'human_bottom',
                                      'pic_bottom']

        # 记录图片每一步的裁剪流程
        # 具体图片裁剪信息，在裁剪过程中不断计算，更新
        self.set_top_name = None
        self.set_bottom_name = None
        self.set_crop_parameter = None

        self.crop_top_site = None
        self.crop_top_name = None
        self.crop_bottom_site = None
        self.crop_bottom_name = None

        self.human_box = None
        self.cloth_box = None

    def crop(self, slot_config, img):
        """
        需求文档： https://www.teambition.com/task/619e01f619f251003f289802
        """
        slot_width = slot_config.get('width')
        slot_height = slot_config.get('height')
        crop_info = slot_config.get(CROP)
        crop_method = slot_config.get(STRETCH, 'fix')

        if img.img_crop_type == GIF:  # 如果分类为动图，特殊处理
            return self.__gif_crop(slot_width, slot_height, img)

        self.si.get_slot_and_img_ration(slot_width / slot_height, img.ratio)

        # 是否需要裁剪
        need_crop = crop_info.get(NEED_CROP)
        if not need_crop:
            self.si.get_crop_site([None, None], [None, None])
            self.si.expected_crop_result()
            return self._need_center_crop(crop_method, slot_width, slot_height, img)

        # 槽是否设置对应图片类型的裁剪规则
        crop_parameter = crop_info.get(img.img_crop_type, {})
        if not crop_parameter:
            self.si.get_crop_site([None, None], [None, None])
            self.si.expected_crop_result()
            # print(f"居中裁剪： 未获取到裁剪参数，图片类型为 {img.image_classification}")
            return self._need_center_crop(crop_method, slot_width, slot_height, img)

        # 资质图裁剪
        # if img.img_crop_type == BRAND:
        #     self.si.get_crop_site([None, None], [None, None])
        #     return self.brand_crop(img, slot_config, slot_width, slot_height)

        # 是否配置裁剪主商品
        if crop_info.get(MAIN_PRODUCT_CROP) == 1:  # 需要裁剪主商品
            img.deal_crop_key_point(main_product_crop=True)

        # 左右居中方式  human_center human_and_cloth brand
        x_center_method = crop_info.get(X_CENTER_METHOD, "human_center")

        # 寻找上下裁剪点
        top, bottom, left, right = self._get_top_bottom_crop_point(crop_parameter, img, x_center_method, crop_method)

        # 计算 内边距
        padding_unit = crop_parameter.get(PADDING_UNIT, 'pi')  # 'percent pe' 'pixel pi'
        if padding_unit == 'pe':  # 简写
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) * 0.01, float(
                crop_parameter.get(RIGHT, 0)) * 0.01
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) * 0.01, float(
                crop_parameter.get(BOTTOM, 0)) * 0.01
        else:
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) / slot_width, float(
                crop_parameter.get(RIGHT, 0)) / slot_width
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) / slot_height, float(
                crop_parameter.get(BOTTOM, 0)) / slot_height

        x1 = max(left - padding_left, 0)
        x2 = min(right + padding_right, 1)
        y1 = max(top - padding_top, 0)
        y2 = min(bottom + padding_bottom, 1)

        self.si.get_top_and_bottom_padding(top, padding_top, bottom, padding_bottom)

        if crop_method == 'w':  # 自适应裁剪，slot宽不变,高可以改变
            # 保留上下 y ，还是保留左右居中 x
            retain_x_or_y = crop_info.get(RETAIN_X_OR_Y, "y")
            if retain_x_or_y == 'y':
                # 旧版逻辑
                # 需要改变slot大小
                width_scale = slot_width / ((x2 - x1) * img.width)
                transform_img_height = width_scale * img.height
                scale_x = scale_y = width_scale
                x = x1
                y = y1
                crop_width = slot_width / scale_x
                crop_height = ((y2 - y1) * img.height)
                amend_layout_height = (y2 - y1) * transform_img_height - slot_height
            else:
                # 新逻辑
                height_scale = slot_height / ((y2 - y1) * img.height)
                transform_img_width = img.width * height_scale
                x_center = left + (right - left) / 2  # x 轴的居中
                center_ = x_center * transform_img_width

                # 在图片的范围内
                tag_1 = center_ - slot_width / 2
                tag_2 = center_ + slot_width / 2
                if tag_1 >= 0 and tag_2 <= transform_img_width:  # 两边都满足，直接居中裁剪
                    scale_x = scale_y = height_scale
                    y = y1
                    x = (center_ - slot_width / 2) / transform_img_width
                    crop_width = slot_width / scale_x
                    crop_height = slot_height / scale_x
                    amend_layout_height = 0
                else:
                    # 需要改变slot大小
                    min_x = min(x_center, 1 - x_center)
                    width_scale = slot_width / (2 * min_x * img.width)
                    transform_img_height = width_scale * img.height
                    scale_x = scale_y = width_scale
                    x = x_center - min_x
                    y = y1
                    crop_width = slot_width / scale_x
                    crop_height = ((y2 - y1) * img.height)
                    amend_layout_height = (y2 - y1) * transform_img_height - slot_height

            crop_x = x * img.width
            crop_y = y * img.height

        elif crop_method == 'e':  # 扩边裁剪，slot高不变,宽可以改变
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.__extension_crop(
                y1, y2, {'x1': x1, 'x2': x2}, [slot_width, slot_height], img
            )

        else:  # 固定裁剪, slot宽高不变
            # 裁剪高铺满的情况下,等比放缩 裁剪 宽

            height_scale = slot_height / ((y2 - y1) * img.height)
            transform_img_width = img.width * height_scale

            if transform_img_width >= slot_width:  # 图片宽度大于槽的宽度
                scale_x = scale_y = height_scale
                x_center = left + (right - left) / 2  # x 轴的居中
                y = y1
                x = self._alternative_center(x_center, slot_width, transform_img_width)

                self.si.main_part_offset(x * img.width + slot_width / scale_x / 2, x_center * img.width, 'x')
                self.si.expected_crop_result()

            else:
                width_scale = slot_width / img.width
                y = y1
                # 保留上下，还是保留左右  human_center human_and_cloth brand
                retain_x_or_y = crop_info.get(RETAIN_X_OR_Y, "x")
                if retain_x_or_y == "y":
                    scale_x = scale_y = width_scale
                    x = x1
                    y_center = y1 + (y2 - y1) / 2
                    self.si.main_part_offset(y1 * img.height + slot_height / scale_x / 2, y_center * img.height, 'y')
                else:
                    left_ = left
                    right_ = 1 - right
                    bottom_ = 1 - y1
                    min_x = min(left_, right_, bottom_)  # 找到最小的距离
                    # 同比缩放图片，并计算裁剪点x 的数值
                    if bottom_ > min_x and left <= right:
                        width_scale = slot_width / ((right + min_x) * img.width)
                        x = 0
                    else:
                        width_scale = slot_width / ((1 - (left - min_x)) * img.width)
                        x = left - min_x
                    scale_x = scale_y = width_scale
                    self.si.extension()
                    self.si.crop_head()
                    self.si.main_part_offset(0.5, 0.5, 'y')

            crop_x = x * img.width
            crop_y = y * img.height
            crop_width = slot_width / scale_x
            crop_height = slot_height / scale_x
            amend_layout_height = 0

        self.si.img_scale_ratio(scale_x, scale_y)

        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def crop_details(self, img, slot_config, slot_width, slot_height, best_image_dict):
        """
        裁剪细节图 需求文档
        https://thoughts.teambition.com/workspaces/5e37b6adf0f331001a9c172d/docs/607e874e4cc5830001d956c7
        """
        top_site = best_image_dict['top_site']
        bottom_site = best_image_dict['bottom_site']

        crop_info = slot_config.get(CROP)
        crop_dict = {
            "need_crop": crop_info.get(NEED_CROP),
            "img_self-adaption": slot_config.get(STRETCH, 'fix'),
            "layout_self-adaption": 'false',
            "crop_method": crop_info.get(CROP_DETAIL, {}),
            "match_slot": best_image_dict['match_s']
        }

        # 每个槽设置的裁剪参数
        crop_parameter = crop_dict.get('crop_method')

        crop_top_site = min(top_site[1], bottom_site[1])
        crop_bottom_site = max(top_site[1], bottom_site[1])
        left = min(top_site[0], bottom_site[0])
        right = max(top_site[0], bottom_site[0])

        # 需要裁剪、加上 内边距
        # 计算 高的内边距
        padding_unit = crop_parameter.get(PADDING_UNIT, 'pi')  # 'percent pe' 'pixel pi'
        if padding_unit == 'pe':  # 简写
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) * 0.01, float(
                crop_parameter.get(RIGHT, 0)) * 0.01
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) * 0.01, float(
                crop_parameter.get(BOTTOM, 0)) * 0.01
        else:
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) / slot_width, float(
                crop_parameter.get(RIGHT, 0)) / slot_width
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) / slot_height, float(
                crop_parameter.get(BOTTOM, 0)) / slot_height
        # 加上内边界后 裁剪点 x1,y1,x2,y2
        x1 = max(left - padding_left, 0)
        x2 = min(right + padding_right, 1)

        y1 = max(crop_top_site - padding_top, 0)
        y2 = min(crop_bottom_site + padding_bottom, 1)
        print(f"{crop_dict['match_slot']} {x1},{y1},{x2},{y2} {img.name_slot}{img.name_slot_index}")
        get_crop_point_result_dict = dict()
        get_crop_point_result_dict['x1'] = x1
        get_crop_point_result_dict['x2'] = x2

        if crop_dict['img_self-adaption'] == 'w':
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = \
                self.self_adaption_height_crop(y1, y2, get_crop_point_result_dict, [slot_width, slot_height], img)
            if crop_dict.get('layout_self-adaption') == 'false':
                amend_layout_height = 0
        else:
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.img_crop_detail_crop(
                y1, y2, x1, x2, [slot_width, slot_height], img)

        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def center_crop(self, origin_slot_width, origin_slot_height, img_width, img_height):
        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / img_height
        transform_img_width = img_width * height_scale
        if transform_img_width >= origin_slot_width:  # 图片宽度大于槽的宽度

            scale_x = scale_y = height_scale
            y = 0
            x = 0.5 * transform_img_width - origin_slot_width / 2
            self.si.main_part_offset(0.5, 0.5, 'x')
        else:
            # 宽铺满的情况下,等比放缩高  裁剪高
            width_scale = origin_slot_width / img_width
            scale_x = scale_y = width_scale
            transform_img_height = width_scale * img_height
            x = 0
            y = 0.5 * transform_img_height - origin_slot_height / 2
            self.si.main_part_offset(0.5, 0.5, 'y')

        self.si.img_scale_ratio(scale_x, scale_y)
        return x / scale_x, y / scale_x, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0, img_width, img_height

    def self_adaption_height_crop(self, top, bottom, crop_point_result_dict, origin_slot, img=None, img_width=None,
                                  img_height=None):
        # img_self-adaption: true,false     图片自适应
        # layout_self-adaption: true,false  画布自适应

        if img:
            img_width = img.width
            img_height = img.height
        # 宽铺满的情况下,等比放缩 计算 高  裁剪高
        origin_slot_width, origin_slot_height = origin_slot
        # 加上内边界后 裁剪点 x1,x2
        x1 = crop_point_result_dict['x1']
        x2 = crop_point_result_dict['x2']
        width_scale = origin_slot_width / ((x2 - x1) * img_width)
        transform_img_height = width_scale * img_height
        scale_x = scale_y = width_scale
        x = x1
        y = top
        crop_width = origin_slot_width / scale_x
        crop_height = ((bottom - top) * img_height)

        amend_layout_height = (bottom - top) * transform_img_height - origin_slot_height

        self.si.img_scale_ratio(scale_x, scale_y)
        # print(f'自适应裁剪: slot高度{origin_slot_height} -> {amend_layout_height}')
        return x * img_width, y * img_height, crop_width, crop_height, scale_x, scale_y, amend_layout_height

    def _need_center_crop(self, crop_method, slot_width, slot_height, img):
        """裁剪所需信息不全，居中裁剪"""
        if crop_method == 'w':  # 宽固定，高自适应
            get_crop_point_result_dict = {'x1': 0, 'x2': 1}
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self. \
                self_adaption_height_crop(0, 1, get_crop_point_result_dict, [slot_width, slot_height], img)
            amend_layout_height = 0  # 历史遗留问题，留着就行
            return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height
        else:
            return self.center_crop(slot_width, slot_height, img.width, img.height)

    def __extension_crop(self, top, bottom, crop_point_result_dict, origin_slot, img):
        # 一定要考虑图片放缩 后 相关点的变化

        origin_slot_width, origin_slot_height = origin_slot
        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / ((bottom - top) * img.height)
        transform_img_width = img.width * height_scale

        scale_x = scale_y = height_scale
        y = top
        # 加上padding的 左右点
        x1 = crop_point_result_dict['x1']
        x2 = crop_point_result_dict['x2']
        center = x1 + (x2 - x1) / 2

        left = center * transform_img_width - origin_slot_width / 2  # slot左边 距离 原点的位置
        right = center * transform_img_width + origin_slot_width / 2  # slot右边 距离 原点的位置

        tag_1 = (left >= 0)
        tag_2 = (right <= transform_img_width)

        if not (tag_1 and tag_2):  # 一个不满足就需要扩边
            self.si.extension()
        self.si.expected_crop_result()

        x = (center * transform_img_width - origin_slot_width / 2)
        # 调整 左右 扩边留白相等
        if tag_1 and tag_2:  # 左右 都有足够 预留范围 不用调整
            pass
        elif tag_1 and not tag_2:  # 左范围够 右范围不够 # 尝试 向 左移动
            if (right - transform_img_width) <= left:  # 右扩边 小于 左距离
                x = left - (right - transform_img_width)
            else:
                x = -((right - transform_img_width - left) / 2)
        elif not tag_1 and tag_2:  # 左范围不够 右范围够
            if -left <= (transform_img_width - right):
                x = 0
            else:
                x = (left + (transform_img_width - right)) / 2
        else:  # 左右范围都不够
            # 左右白边相加 求平均值
            x = -((right - transform_img_width) - left) / 2
        x = x / transform_img_width

        self.si.main_part_offset(x * img.width + origin_slot_width / scale_x / 2, center * img.width, 'x')

        self.si.img_scale_ratio(scale_x, scale_y)
        return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0

    def __gif_crop(self, slot_width, slot_height, img):
        """动图宽固定，高自适应 计算出 裁剪参数"""
        crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self. \
            self_adaption_height_crop(0, 1, {'x1': 0, 'x2': 1}, [slot_width, slot_height], img)
        amend_layout_height = 0
        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def _get_top_bottom_crop_point(self, crop_parameter, img, x_center_method, crop_method):
        #  寻找 裁剪点

        if img.img_crop_type == BRAND:  # 资质图
            crop_top_site = img.brand_crop_point['brand_top']['top_site']
            crop_bottom_site = img.brand_crop_point['brand_bottom']['bottom_site']

            crop_top = crop_top_site[1]
            crop_bottom = crop_bottom_site[1]
            crop_left = crop_top_site[0]
            crop_right = crop_bottom_site[0]
            self.si.get_crop_site(['brand_top', 'brand_bottom'], ['brand_top', 'brand_bottom'])

        elif img.img_crop_type == TILE:  # 平铺图
            crop_top_site = img.crop_point.get('garment_top',
                                               {'top_site': [0, 0], 'bottom_site': [1, 1]})['top_site']
            crop_bottom_site = img.crop_point.get('garment_bottom',
                                                  {'top_site': [0, 0], 'bottom_site': [1, 1]})['bottom_site']

            crop_top = crop_top_site[1]
            crop_bottom = crop_bottom_site[1]
            crop_left = crop_top_site[0]
            crop_right = crop_bottom_site[0]
            self.si.get_crop_site(['garment_top', 'garment_bottom'], ['garment_top', 'garment_bottom'])
        else:  # 模特图
            top_site = crop_parameter.get(TOP_SITE)
            bottom_site = crop_parameter.get(BOTTOM_SITE)
            # 取裁剪点/兼容的裁剪点
            #  Todo 裁剪点 顺序

            crop_top_site, crop_top_name = self._from_img_get_top_site(img.crop_point, top_site)
            crop_bottom_site, crop_bottom_name = self._from_img_get_bottom_site(img.crop_point, bottom_site)

            self.si.get_crop_site([top_site, bottom_site], [crop_top_name, crop_bottom_name])

            crop_top = crop_top_site[1]
            crop_bottom = crop_bottom_site[1]

            # 根据设置居中类型，选择左右点. 左右居中方式  human_center human_and_cloth brand
            if x_center_method == "human_center":
                # 计算居中点
                human_center = img.humans.calculate_x_human_center(crop_top, crop_bottom)

                # 人体X轴中线未找到，则取人和衣服X轴最靠外的点
                if human_center is None:
                    crop_left, crop_right = self._get_x_human_and_cloth(img)
                else:
                    crop_left = crop_right = human_center

            elif x_center_method == "human_and_cloth":

                crop_left, crop_right = self._get_x_human_and_cloth(img)
            else:
                crop_left = crop_right = 0.5

        # 对上下左右点进行纠正
        top = min(crop_top, crop_bottom)
        bottom = max(crop_top, crop_bottom)
        left = min(crop_left, crop_right)
        right = max(crop_left, crop_right)

        return top, bottom, left, right

    @staticmethod
    def _get_x_human_and_cloth(img):
        """左右取人体和衣服最靠外的点"""
        human_box_top = img.crop_point.get('human_top',
                                           {'top_site': [0, 0], 'bottom_site': [1, 1]})['top_site']
        human_box_bottom = img.crop_point.get('human_bottom',
                                              {'top_site': [0, 0], 'bottom_site': [1, 1]})['bottom_site']
        cloth_box_top = img.crop_point.get('garment_top',
                                           {'top_site': [0, 0], 'bottom_site': [1, 1]})['top_site']
        cloth_box_bottom = img.crop_point.get('garment_bottom',
                                              {'top_site': [0, 0], 'bottom_site': [1, 1]})['bottom_site']

        crop_left = min(human_box_top[0], human_box_bottom[0], cloth_box_top[0], cloth_box_bottom[0])
        crop_right = max(human_box_top[0], human_box_bottom[0], cloth_box_top[0], cloth_box_bottom[0])
        return crop_left, crop_right

    def _from_img_get_top_site(self, crop_point, top_site):
        if top_site not in self.crop_sequence:
            flag_index = self.garment_crop_sequence.index(top_site)
            crop_top, crop_top_name = None, None
            while flag_index >= 0:
                crop_top, crop_top_name = crop_point.get(self.garment_crop_sequence[flag_index]), \
                                          self.garment_crop_sequence[flag_index]
                if crop_top:
                    break
                flag_index -= 1
            return crop_top['top_site'], crop_top_name

        flag_index = self.crop_sequence.index(top_site)
        crop_top, crop_top_name = None, None
        while flag_index >= 0:
            crop_top, crop_top_name = crop_point.get(self.crop_sequence[flag_index]), self.crop_sequence[flag_index]
            if crop_top:
                break
            flag_index -= 1
        return crop_top['top_site'], crop_top_name

    def _from_img_get_bottom_site(self, crop_point, bottom_site):
        if bottom_site not in self.crop_sequence:
            flag_index = self.garment_crop_sequence.index(bottom_site)
            crop_bottom, crop_bottom_name = None, None
            while flag_index < len(self.crop_sequence):
                crop_bottom, crop_bottom_name = crop_point.get(self.garment_crop_sequence[flag_index]), \
                                                self.garment_crop_sequence[
                                                    flag_index]
                if crop_bottom:
                    break
                flag_index += 1
            return crop_bottom['bottom_site'], crop_bottom_name

        flag_index = self.crop_sequence.index(bottom_site)
        crop_bottom, crop_bottom_name = None, None
        while flag_index < len(self.crop_sequence):
            crop_bottom, crop_bottom_name = crop_point.get(self.crop_sequence[flag_index]), self.crop_sequence[
                flag_index]
            if crop_bottom:
                break
            flag_index += 1
        return crop_bottom['bottom_site'], crop_bottom_name

    def _alternative_center(self, center, slot_height_or_width, transform_height_or_width):
        center_ = center * transform_height_or_width
        # 在图片的范围内
        tag_1 = (center_ - slot_height_or_width / 2) >= 0
        tag_2 = (center_ + slot_height_or_width / 2) <= transform_height_or_width

        if tag_1 and tag_2:  # 上下 都有足够 预留范围
            x_or_y = center_ - slot_height_or_width / 2
        elif tag_1 and not tag_2:  # 1范围够 2范围不够
            x_or_y = transform_height_or_width - slot_height_or_width
        elif tag_2 and not tag_1:  # 1范围不够 2范围够
            x_or_y = 0
        else:  # 未知情况
            x_or_y = 0.5 * transform_height_or_width - slot_height_or_width / 2
        x_or_y = x_or_y / transform_height_or_width

        if not (tag_1 and tag_2):
            self.si.extension()

        return x_or_y

    def img_crop_detail_crop(self, top, bottom, left, right, origin_slot, img):
        """图片裁剪出 细节图"""

        origin_slot_width, origin_slot_height = origin_slot

        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / ((bottom - top) * img.height)
        transform_img_width = img.width * height_scale

        if transform_img_width >= origin_slot_width:  # 图片宽度大于槽的宽度
            scale_x = scale_y = height_scale
            center = left + (right - left) / 2
            y = top
            x = self._alternative_center(center, origin_slot_width, transform_img_width)
            self.si.main_part_offset(0.5, 0.5, 'x')
        else:
            # 宽铺满的情况下,等比放缩 计算 高  裁剪高

            # 加上内边界后 裁剪点 x1,x2
            x1 = left
            x2 = right

            width_scale = origin_slot_width / ((x2 - x1) * img.width)
            # width_scale = origin_slot_width / img.width
            transform_img_height = width_scale * img.height
            scale_x = scale_y = width_scale
            x = x1  # 左位置为  加上内边界后 的左
            # x = 0  # 左位置为  加上内边界后 的左
            center = top + (bottom - top) / 2
            y = self._alternative_center(center, origin_slot_height, transform_img_height)
            self.si.main_part_offset(0.5, 0.5, 'y')
        self.si.img_scale_ratio(scale_x, scale_y)
        return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0



# 标签  每个元素
ES = 'es'  # 每个元素
# 属性
T = 't'  # type 类型 image text table|  ms中的 标题 | ms-sub 中的 标题
SLOT = 's'  # slot配置
URL = 'u'  # 链接
WIDTH = 'w'  # 图片宽
HEIGHT = 'h'  # 图片高

# 匹配状态
MATCH_STATUS = 'm_s'  # 状态: f：未匹配成功 s：匹配成功

# 模块标签
MS = 'ms'  # 画布、模块  可以商量改一下 ，改为画布
MS_LOOP = 'ms-loop'
# 子模块标签
MS_SUB = 'ms-sub'  # 子画布
DATA_COLOR = 'data-color'  # 3.0版本不支持，4.0支持颜色排版
# 属性
LOOP = 'l'  # 循环的类型

#  Slot 配置的 mapping
VERSION = 'v'
SLOT_VERSION = '2020-12-12'
UUID = 'id'
MATCH_SLOT = 'ms'  # ai匹配
NAME_SLOT = 'ns'  # 命名替换
MATCH_UUID = 'm_u'  # 匹配uuid
REPEAT_MATCH_UUID = 'r_m_u'  # 重复匹配uuid

DEFAULT_DISPLAY = 'd_d'  # 未替换时，默认展示的图片 0系统占位图  1模板图片 2空白图片

COLOR = 'c'  # 颜色
REPEAT_SLOT_IMG = 'r_s_i'  # 槽内图片是否被其他槽重复使用1次
REPEAT_SLOT_IMG_INFINITE = 'r_s_i_in'  # 槽内图片是否被其他槽重复使用N次
REPEAT = 'r'  # 重复替换
STRETCH = 's'  # 是否自适应 fix固定不变  w自适应
CROP = 'cr'  # 裁剪
NEED_CROP = 'n_c'  # 0不裁剪  1 裁剪
X_CENTER_METHOD = "x_c_m"  # 左右居中方式  human_center human_and_cloth brand
RETAIN_X_OR_Y = "r_x_o_y"  # # 保留上下 y 保留左右居中 x
MAIN_PRODUCT_CROP = "m_p_c"  # 0不裁剪主商品  1 裁剪主商品
MODEL_TOP = 'm_t'
MODEL_BUST = 'm_bu'
MODEL_BOTTOM = 'm_bo'
MODEL_FULL = 'm_f'
TILE = 't'
BRAND = 'b'
CROP_DETAIL = 'c_d'
GIF = 'g'

TOP_SITE = 'ts'
BOTTOM_SITE = 'bs'
TOP = 't'
LEFT = 'l'
BOTTOM = 'b'
RIGHT = 'r'
PADDING_UNIT = 'pU'  # pe pi

DROP = 'd'
TYPE = 't'

DATA_SLOT_ID = 'data-slot-id'  # 元素新增uuid 与slots_json 绑定
DATA_TEMPLATE_COLOR = 'data-template-color'  # 模板背景色色块  3.0版本不支持，4.0支持颜色排版



from .html_slot_config import *


class ShouldIndex:
    def __init__(self):
        # 裁剪指标
        self.crop_indexing_dict = {
            'extension': {'status': False},
            'main_part_offset': {'coordinates': 'x', 'offset': 0},
            'crop_head': {'status': False},
            'get_crop_point': {'status': False},
            'slot_and_img_ration': {'offset': 1},
        }

    def extension(self):
        self.crop_indexing_dict['extension'] = {
            'status': True
        }

    def main_part_offset(self, pic_center, main_part_center, coordinates):
        """
        图片主体内容 偏移 图片中心的比率
        :param pic_center:  裁剪后中点
        :param main_part_center: 主题中点
        :param coordinates:  x 轴 或 y轴 ,原点在左上方处
        :return:
        """

        if pic_center < main_part_center:
            offset = - (main_part_center - pic_center) / pic_center
        elif pic_center == main_part_center:
            offset = 0
        else:
            offset = (pic_center - main_part_center) / pic_center

        self.crop_indexing_dict['main_part_offset'] = {
            'coordinates': coordinates,
            'offset': offset
        }

        return offset

    def img_scale_ratio(self, width_scale, height_scale):
        """
        图片缩放比率 ,目前图片都是等比放缩。所以 宽高缩放率是一样的
        小于1  为缩放
        大于1  为放大
        :param width_scale:
        :param height_scale:
        :return:
        """
        self.crop_indexing_dict['img_scale_ratio'] = {
            'width_scale': width_scale,
            'height_scale': height_scale,
        }

        return width_scale

    def get_slot_and_img_ration(self, slot_ratio, crop_main_part_ratio):
        """
        slot宽高比 / 裁剪主题宽高比
        :param slot_ratio:
        :param crop_main_part_ratio:
        :return:
        """
        self.crop_indexing_dict['slot_and_img_ration'] = {
            'slot_ratio': slot_ratio,
            'img_ratio': crop_main_part_ratio,
            'offset': abs(crop_main_part_ratio - slot_ratio) / slot_ratio
        }
        return None

    def get_crop_site(self, origin_crop_site, crop_site):
        """预设裁剪点是否取到 或 兼容取到"""

        self.crop_indexing_dict['get_crop_point'] = {
            'origin_crop_site': origin_crop_site,
            'crop_site': crop_site,
            'status': True if origin_crop_site == crop_site else False,
        }

    def crop_head(self):
        """裁剪头"""
        self.crop_indexing_dict['crop_head'] = {
            'status': True
        }


class ImageCrop:
    def __init__(self):
        self.si = ShouldIndex()
        #  Todo 裁剪点 顺序
        self.crop_sequence = ['pic_top', 'human_top', 'eye', 'nose', 'ear', 'mouth', 'mandible', 'shoulder', 'elbow',
                              'wrist',"waist", 'hip', 'knee', 'ankle', 'human_bottom', 'pic_bottom']
        self.garment_crop_sequence = ['pic_top', 'human_top', 'garment_top', 'garment_bottom', 'human_bottom',
                                      'pic_bottom']

    def gif_crop(self, slot_width, slot_height, img):
        """动图宽固定，高自适应 计算出 裁剪参数"""
        crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self. \
            self_adaption_height_crop(0, 1, {'x1': 0, 'x2': 1}, [slot_width, slot_height], img)
        amend_layout_height = 0
        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def brand_crop(self, img, slot_config, slot_width, slot_height):
        """资质图裁剪"""
        top_site = img.brand_crop_point['brand_top']['top_site']
        bottom_site = img.brand_crop_point['brand_bottom']['bottom_site']

        crop_info = slot_config.get(CROP)
        crop_dict = {
            "need_crop": crop_info.get(NEED_CROP),
            "img_self-adaption": slot_config.get(STRETCH, 'fix'),
            "layout_self-adaption": 'false',
            "crop_method": crop_info.get(BRAND, {})
        }

        # 每个槽设置的裁剪参数
        crop_parameter = crop_dict.get('crop_method')

        crop_top_site = min(top_site[1], bottom_site[1])
        crop_bottom_site = max(top_site[1], bottom_site[1])
        left = min(top_site[0], bottom_site[0])
        right = max(top_site[0], bottom_site[0])

        # 需要裁剪、加上 内边距
        # 计算 高的内边距
        padding_unit = crop_parameter.get(PADDING_UNIT, 'pi')  # 'percent pe' 'pixel pi'
        if padding_unit == 'pe':  # 简写
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) * 0.01, float(
                crop_parameter.get(RIGHT, 0)) * 0.01
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) * 0.01, float(
                crop_parameter.get(BOTTOM, 0)) * 0.01
        else:
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) / slot_width, float(
                crop_parameter.get(RIGHT, 0)) / slot_width
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) / slot_height, float(
                crop_parameter.get(BOTTOM, 0)) / slot_height
        # 加上内边界后 裁剪点 x1,y1,x2,y2
        x1 = max(left - padding_left, 0)
        x2 = min(right + padding_right, 1)

        y1 = max(crop_top_site - padding_top, 0)
        y2 = min(crop_bottom_site + padding_bottom, 1)

        get_crop_point_result_dict = dict()
        get_crop_point_result_dict['x1'] = x1
        get_crop_point_result_dict['x2'] = x2

        if crop_dict['img_self-adaption'] == 'w':
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.self_adaption_height_crop(
                y1, y2, get_crop_point_result_dict, [slot_width, slot_height], img)
            if crop_dict.get('layout_self-adaption') == 'false':
                amend_layout_height = 0
        else:
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.img_crop_detail_crop(
                y1, y2, x1, x2, [slot_width, slot_height], img)

        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def crop(self, slot_config, img):
        """
        裁剪逻辑 需求文档
        https://thoughts.teambition.com/workspaces/5e37b6adf0f331001a9c172d/docs/5f2e438de6eed50001359150
        """
        slot_width = slot_config.get('width')
        slot_height = slot_config.get('height')

        if img.image_classification == '动图':  # 如果分类为动图，特殊处理
            return self.gif_crop(slot_width, slot_height, img)

        crop_dict = self.get_crop_dict(slot_config, img)
        img_crop_type = crop_dict['img_crop_type']

        # if 'asset/1000175/47178d5e-6db6-4f7e-b7f8-f1b284476e26.jpg' in img.storage_key:
        #     print(12312323)

        self.si.get_slot_and_img_ration(slot_width / slot_height, img.ratio)

        # 是否需要裁剪
        need_crop = crop_dict['need_crop']
        if not need_crop:
            # print("居中裁剪, 图片设置为不裁剪，")
            self.si.get_crop_site([None, None], [None, None])
            return self.need_center_crop(crop_dict, slot_width, slot_height, img)

        # 每个槽设置的裁剪参数
        crop_parameter = crop_dict.get('crop_method')
        if not crop_parameter:
            self.si.get_crop_site([None, None], [None, None])
            # print(f"居中裁剪： 未获取到裁剪参数，图片类型为 {img.image_classification}")
            return self.need_center_crop(crop_dict, slot_width, slot_height, img)

        # 资质图裁剪
        if img_crop_type == BRAND:
            self.si.get_crop_site([None, None], [None, None])
            return self.brand_crop(img, slot_config, slot_width, slot_height)

        # 找到裁剪点
        get_crop_point_result_dict = self.get_crop_point(crop_parameter, img, img_crop_type)
        if not get_crop_point_result_dict:
            return self.need_center_crop(crop_dict, slot_width, slot_height, img)

        crop_top_site, crop_bottom_site = get_crop_point_result_dict['crop_top_site'], get_crop_point_result_dict[
            'crop_bottom_site']

        if get_crop_point_result_dict['crop_top_name'] == 'pic_top' and get_crop_point_result_dict[
            'crop_bottom_name'] == 'pic_bottom' and crop_dict['img_self-adaption'] != 'e':
            # 居中裁剪 并往上移动
            # print(123123123123)
            return self.need_pic_top_and_pic_bottom_crop(crop_dict, slot_width, slot_height, img,
                                                         get_crop_point_result_dict)

        left = get_crop_point_result_dict['crop_left']
        right = get_crop_point_result_dict['crop_right']

        # 计算 高的内边距
        padding_unit = crop_parameter[PADDING_UNIT]  # 'percent pe' 'pixel pi'
        if padding_unit == 'pe':  # 简写
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) * 0.01, float(
                crop_parameter.get(RIGHT, 0)) * 0.01
            padding_top, padding_bottom = float(crop_parameter[TOP]) * 0.01, float(crop_parameter[BOTTOM]) * 0.01
        else:
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) / slot_width, float(
                crop_parameter.get(RIGHT, 0)) / slot_width
            padding_top, padding_bottom = float(crop_parameter[TOP]) / slot_height, float(
                crop_parameter[BOTTOM]) / slot_height
        # 加上内边界后 裁剪点 x1,y1,x2,y2
        x1 = max(left - padding_left, 0)
        x2 = min(right + padding_right, 1)
        get_crop_point_result_dict['x1'] = x1
        get_crop_point_result_dict['x2'] = x2
        y1 = max(crop_top_site[1] - padding_top, 0)
        y2 = min(crop_bottom_site[1] + padding_bottom, 1)

        if crop_dict['img_self-adaption'] == 'w':
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.self_adaption_height_crop(
                y1, y2, get_crop_point_result_dict, [slot_width, slot_height], img)
            if crop_dict.get('layout_self-adaption') == 'false':
                amend_layout_height = 0
        elif crop_dict['img_self-adaption'] == 'e':
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.extension_crop(
                y1, y2, get_crop_point_result_dict, [slot_width, slot_height], img, img_crop_type
            )
        else:
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.height_crop(
                y1, y2, get_crop_point_result_dict, [slot_width, slot_height], img, img_crop_type
            )

        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def crop_details(self, img, slot_config, slot_width, slot_height, best_image_dict):
        """
        裁剪细节图 需求文档
        https://thoughts.teambition.com/workspaces/5e37b6adf0f331001a9c172d/docs/607e874e4cc5830001d956c7
        """
        top_site = best_image_dict['top_site']
        bottom_site = best_image_dict['bottom_site']

        crop_info = slot_config.get(CROP)
        crop_dict = {
            "need_crop": crop_info.get(NEED_CROP),
            "img_self-adaption": slot_config.get(STRETCH, 'fix'),
            "layout_self-adaption": 'false',
            "crop_method": crop_info.get(CROP_DETAIL, {}),
            "match_slot": best_image_dict['match_s']
        }

        # 每个槽设置的裁剪参数
        crop_parameter = crop_dict.get('crop_method')

        crop_top_site = min(top_site[1], bottom_site[1])
        crop_bottom_site = max(top_site[1], bottom_site[1])
        left = min(top_site[0], bottom_site[0])
        right = max(top_site[0], bottom_site[0])

        # 需要裁剪、加上 内边距
        # 计算 高的内边距
        padding_unit = crop_parameter.get(PADDING_UNIT, 'pi')  # 'percent pe' 'pixel pi'
        if padding_unit == 'pe':  # 简写
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) * 0.01, float(
                crop_parameter.get(RIGHT, 0)) * 0.01
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) * 0.01, float(
                crop_parameter.get(BOTTOM, 0)) * 0.01
        else:
            padding_left, padding_right = float(crop_parameter.get(LEFT, 0)) / slot_width, float(
                crop_parameter.get(RIGHT, 0)) / slot_width
            padding_top, padding_bottom = float(crop_parameter.get(TOP, 0)) / slot_height, float(
                crop_parameter.get(BOTTOM, 0)) / slot_height
        # 加上内边界后 裁剪点 x1,y1,x2,y2
        x1 = max(left - padding_left, 0)
        x2 = min(right + padding_right, 1)

        y1 = max(crop_top_site - padding_top, 0)
        y2 = min(crop_bottom_site + padding_bottom, 1)
        print(f"{crop_dict['match_slot']} {x1},{y1},{x2},{y2} {img.name_slot}{img.name_slot_index}")
        get_crop_point_result_dict = dict()
        get_crop_point_result_dict['x1'] = x1
        get_crop_point_result_dict['x2'] = x2

        if crop_dict['img_self-adaption'] == 'w':
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.self_adaption_height_crop(
                y1, y2, get_crop_point_result_dict, [slot_width, slot_height], img)
            if crop_dict.get('layout_self-adaption') == 'false':
                amend_layout_height = 0
        else:
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self.img_crop_detail_crop(
                y1, y2, x1, x2, [slot_width, slot_height], img)

        return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height

    def self_adaption_height_crop(self, top, bottom, crop_point_result_dict, origin_slot, img=None, img_width=None,
                                  img_height=None):
        ''' img_self-adaption: true,flase     图片自适应
            layout_self-adaption: true,flase  画布自适应
        '''
        if img:
            img_width = img.width
            img_height = img.height
        # 宽铺满的情况下,等比放缩 计算 高  裁剪高
        origin_slot_width, origin_slot_height = origin_slot
        # 加上内边界后 裁剪点 x1,x2
        x1 = crop_point_result_dict['x1']
        x2 = crop_point_result_dict['x2']
        width_scale = origin_slot_width / ((x2 - x1) * img_width)
        transform_img_height = width_scale * img_height
        scale_x = scale_y = width_scale
        x = x1
        y = top
        crop_width = origin_slot_width / scale_x
        crop_height = ((bottom - top) * img_height)

        amend_layout_height = (bottom - top) * transform_img_height - origin_slot_height

        self.si.img_scale_ratio(scale_x, scale_y)
        # print(f'自适应裁剪: slot高度{origin_slot_height} -> {amend_layout_height}')
        return x * img_width, y * img_height, crop_width, crop_height, scale_x, scale_y, amend_layout_height

    @staticmethod
    def get_crop_dict(slot_config, img):
        # print(slot_config)
        crop_info = slot_config.get(CROP)
        crop_dict = {
            "img_crop_type": img.img_crop_type,
            "need_crop": crop_info.get(NEED_CROP),
            "img_self-adaption": slot_config.get(STRETCH, 'fix'),
            "layout_self-adaption": 'false',
            "crop_method": crop_info.get(img.img_crop_type, {})
        }

        if crop_info.get(MAIN_PRODUCT_CROP) == 1:  # 需要裁剪主商品
            img.deal_crop_key_point(main_product_crop=True)

        return crop_dict

    def height_crop(self, top, bottom, crop_point_result_dict, origin_slot, img, img_crop_type):
        # 一定要考虑图片放缩 后 相关点的变化
        cloth_x1, cloth_y1, cloth_x2, cloth_y2 = crop_point_result_dict['cloth_box']

        origin_slot_width, origin_slot_height = origin_slot

        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / ((bottom - top) * img.height)
        transform_img_width = img.width * height_scale

        if transform_img_width >= origin_slot_width:  # 图片宽度大于槽的宽度
            # print('x轴裁剪: 高铺满的slot,原图等比放缩后, 裁剪宽')
            scale_x = scale_y = height_scale
            y = top
            center = cloth_x1 + (cloth_x2 - cloth_x1) / 2
            # 模特图
            crop_left, crop_right = crop_point_result_dict['crop_left'], crop_point_result_dict['crop_right']

            if img_crop_type in ['m_t', 'm_bu', 'm_bo', 'm_f']:
                # 衣服模特最靠外的 左点 右点 不包含内间距
                if (crop_right - crop_left) * transform_img_width <= origin_slot_width:
                    center = crop_left + (crop_right - crop_left) / 2

            x = self.alternative_center_crop(center, origin_slot_width, transform_img_width, [crop_left, crop_right])

            self.si.main_part_offset(x * img.width + origin_slot_width / scale_x / 2, center * img.width, 'x')

        else:
            # print('y轴裁剪: 宽铺满,原图等比放缩后 裁剪高')
            # 宽铺满的情况下,等比放缩 计算 高  裁剪高

            # 加上内边界后 裁剪点 x1,x2
            x1 = crop_point_result_dict['x1']
            x2 = crop_point_result_dict['x2']

            width_scale = origin_slot_width / ((x2 - x1) * img.width)
            # width_scale = origin_slot_width / img.width
            transform_img_height = width_scale * img.height
            scale_x = scale_y = width_scale
            x = x1  # 左位置为  加上内边界后 的左
            # x = 0  # 左位置为  加上内边界后 的左
            center = cloth_y1 + (cloth_y2 - cloth_y1) / 2
            if img_crop_type in ['m_t', 'm_bu', 'm_bo', 'm_f', 't']:
                if (bottom - top) * transform_img_height >= origin_slot_height:  # 图片放不进 slot内

                    y = top  # 默认向上移动,优先显示主体, 距上边的距离
                    # 未添加padding 裁剪主体的 left right 位置  # 加了padding的 是x1 x2 看要用哪个
                    crop_left, crop_right = crop_point_result_dict['crop_left'], crop_point_result_dict['crop_right']

                    left = crop_left  # 距左边的距离
                    right = 1 - crop_right  # 距右边的距离
                    bottom_ = 1 - y  # 距下边的距离
                    min_x = min(left, right, bottom_)  # 找到最小的距离
                    # 同比缩放图片，并计算裁剪点x 的数值
                    if bottom_ > min_x and left <= right:
                        width_scale = origin_slot_width / ((crop_right + min_x) * img.width)
                        x = 0
                    else:
                        width_scale = origin_slot_width / ((1 - (crop_left - min_x)) * img.width)
                        x = crop_left - min_x

                    scale_x = scale_y = width_scale

                    self.si.extension()
                    self.si.crop_head()
                    self.si.img_scale_ratio(scale_x, scale_y)
                    self.si.main_part_offset(0.5, 0.5, 'y')

                    return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, \
                           scale_x, scale_y, 0
                else:
                    center = top + (bottom - top) / 2
            crop_top, crop_bottom = crop_point_result_dict['crop_top_site'][1], \
                                    crop_point_result_dict['crop_bottom_site'][1]
            y = self.alternative_center_crop(center, origin_slot_height, transform_img_height, [crop_top, crop_bottom])

            self.si.main_part_offset(y * img.height + origin_slot_height / scale_x / 2, center * img.height, 'y')

        self.si.img_scale_ratio(scale_x, scale_y)

        return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0

    def alternative_center_crop(self, center, slot_height_or_width, transform_height_or_width, x_or_y):
        center_ = center * transform_height_or_width
        # 在图片的范围内
        tag_1 = (center_ - slot_height_or_width / 2) >= 0
        tag_2 = (center_ + slot_height_or_width / 2) <= transform_height_or_width

        if tag_1 and tag_2:  # 上下 都有足够 预留范围
            x_or_y = center_ - slot_height_or_width / 2
        elif tag_1 and not tag_2:  # 1范围够 2范围不够
            x_or_y = transform_height_or_width - slot_height_or_width
        elif tag_2 and not tag_1:  # 1范围不够 2范围够
            x_or_y = 0
        else:  # 未知情况
            x_or_y = 0.5 * transform_height_or_width - slot_height_or_width / 2
        x_or_y = x_or_y / transform_height_or_width

        if not (tag_1 and tag_2):
            self.si.extension()

        return x_or_y

    def get_crop_point(self, crop_parameter, img, img_crop_type):
        #  寻找 裁剪点 并兼容

        top_site = crop_parameter.get(TOP_SITE, 'garment_top')
        if top_site == 'null':
            top_site = 'garment_top'

        # if 'mandible' == top_site:
        #     print(top_site)

        bottom_site = crop_parameter.get(BOTTOM_SITE, 'garment_bottom')
        if bottom_site == 'null':
            bottom_site = 'garment_bottom'
        # 裁剪 上下 点
        # Todo 取上下裁剪点时，取出 2组 x1,y1 x2,y2  找最靠外的 x与y
        crop_top_site, crop_top_name = self.from_img_get_top_site(img.crop_point, top_site)
        crop_bottom_site, crop_bottom_name = self.from_img_get_bottom_site(img.crop_point, bottom_site)

        # 裁剪范围不存在
        if not (crop_top_site and crop_bottom_site):
            # print(f'居中裁剪,{top_site},{bottom_site} 裁剪点不存在')
            return False
        # 裁剪 左右 点
        human_box_top = list(img.crop_point.get('human_top', {}).get('top_site', [None, None]))
        human_box_bottom = list(img.crop_point.get('human_bottom', {}).get('bottom_site', [None, None]))
        human_box_top.extend(human_box_bottom)
        human_box = human_box_top

        cloth_box_top = list(img.crop_point.get('garment_top', {}).get('top_site', [None, None]))
        cloth_box_bottom = list(img.crop_point.get('garment_bottom', {}).get('bottom_site', [None, None]))
        cloth_box_top.extend(cloth_box_bottom)
        cloth_box = cloth_box_top

        human_x1, human_y1, human_x2, human_y2 = human_box
        cloth_x1, cloth_y1, cloth_x2, cloth_y2 = cloth_box

        # 左点 # 右点 人体识别点 可能为 0 ；当为None 时 证明 不存在数据ai数据
        if (human_x1 is None) and (cloth_x1 is None):
            # 左点 右点 不存在 识别结果
            left = 0
            right = 1
            human_box = [0, 0, 1, 1]
            cloth_box = [0, 0, 1, 1]

        elif cloth_x1 is None and human_x1 is not None:
            left = human_x1
            right = human_x2
            cloth_box = [0, 0, 1, 1]
        elif human_x1 is None and cloth_x1 is not None:
            left = cloth_x1
            right = cloth_x2
            human_box = [0, 0, 1, 1]
        else:
            left = min(human_x1, cloth_x1)
            right = max(human_x2, cloth_x2)

        if img_crop_type == 't':
            left = cloth_box[0]
            right = cloth_box[2]

        self.si.get_crop_site([top_site, bottom_site], [crop_top_name, crop_bottom_name])

        return {
            'crop_top_site': crop_top_site,
            'crop_top_name': crop_top_name,
            'crop_bottom_site': crop_bottom_site,
            'crop_bottom_name': crop_bottom_name,
            'crop_left': left,
            'crop_right': right,
            'human_box': human_box,
            'cloth_box': cloth_box,
            'set_top_site': top_site,
            'set_bottom_site': bottom_site,
        }

    def from_img_get_top_site(self, crop_point, top_site):
        if top_site not in self.crop_sequence:
            flag_index = self.garment_crop_sequence.index(top_site)
            crop_top, crop_top_name = None, None
            while flag_index >= 0:
                crop_top, crop_top_name = crop_point.get(self.garment_crop_sequence[flag_index]), \
                                          self.garment_crop_sequence[flag_index]
                if crop_top:
                    break
                flag_index -= 1
            return crop_top['top_site'], crop_top_name

        flag_index = self.crop_sequence.index(top_site)
        crop_top, crop_top_name = None, None
        while flag_index >= 0:
            crop_top, crop_top_name = crop_point.get(self.crop_sequence[flag_index]), self.crop_sequence[flag_index]
            if crop_top:
                break
            flag_index -= 1
        return crop_top['top_site'], crop_top_name

    def from_img_get_bottom_site(self, crop_point, bottom_site):
        if bottom_site not in self.crop_sequence:
            flag_index = self.garment_crop_sequence.index(bottom_site)
            crop_bottom, crop_bottom_name = None, None
            while flag_index < len(self.crop_sequence):
                crop_bottom, crop_bottom_name = crop_point.get(self.garment_crop_sequence[flag_index]), \
                                                self.garment_crop_sequence[
                                                    flag_index]
                if crop_bottom:
                    break
                flag_index += 1
            return crop_bottom['bottom_site'], crop_bottom_name

        flag_index = self.crop_sequence.index(bottom_site)
        crop_bottom, crop_bottom_name = None, None
        while flag_index < len(self.crop_sequence):
            crop_bottom, crop_bottom_name = crop_point.get(self.crop_sequence[flag_index]), self.crop_sequence[
                flag_index]
            if crop_bottom:
                break
            flag_index += 1
        return crop_bottom['bottom_site'], crop_bottom_name

    def need_center_crop(self, crop_dict, slot_width, slot_height, img):
        """裁剪所需信息不全，居中裁剪"""
        if crop_dict['img_self-adaption'] == 'w':  # 宽固定，高自适应
            get_crop_point_result_dict = {'x1': 0, 'x2': 1}
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self. \
                self_adaption_height_crop(0, 1, get_crop_point_result_dict, [slot_width, slot_height], img)
            if crop_dict.get('layout_self-adaption') == 'false':
                amend_layout_height = 0
            return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height
        else:
            return self.center_crop(slot_width, slot_height, img.width, img.height)

    def need_pic_top_and_pic_bottom_crop(self, crop_dict, slot_width, slot_height, img, crop_point_result_dict):
        """用户设的裁剪点:图片顶端到图片底端,做特殊处理"""
        if crop_dict['img_self-adaption'] == 'w':  # 宽固定，高自适应
            get_crop_point_result_dict = {'x1': 0, 'x2': 1}
            crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height = self. \
                self_adaption_height_crop(0, 1, get_crop_point_result_dict, [slot_width, slot_height], img)
            if crop_dict.get('layout_self-adaption') == 'false':
                amend_layout_height = 0
            return crop_x, crop_y, crop_width, crop_height, scale_x, scale_y, amend_layout_height, img.width, img.height
        else:
            return self.pic_top_and_pic_bottom_crop(slot_width, slot_height, img, crop_point_result_dict)

    def pic_top_and_pic_bottom_crop(self, origin_slot_width, origin_slot_height, img, crop_point_result_dict):
        """
        特殊处理
        1.高铺满时，尽量人体居中
        2.宽铺满时，往上移动裁剪框
        """
        img_width = img.width
        img_height = img.height
        cloth_x1, cloth_y1, cloth_x2, cloth_y2 = crop_point_result_dict['cloth_box']

        height_scale = origin_slot_height / img_height
        transform_img_width = img_width * height_scale
        if transform_img_width >= origin_slot_width:  # 图片宽度大于槽的宽度
            scale_x = scale_y = height_scale
            y = 0
            center = cloth_x1 + (cloth_x2 - cloth_x1) / 2
            # 模特图
            crop_left, crop_right = crop_point_result_dict['crop_left'], crop_point_result_dict['crop_right']

            if img.img_crop_type in ['m_t', 'm_bu', 'm_bo', 'm_f']:
                # 衣服模特最靠外的 左点 右点 不包含内间距
                if (crop_right - crop_left) * transform_img_width <= origin_slot_width:
                    center = crop_left + (crop_right - crop_left) / 2

            x = self.alternative_center_crop(center, origin_slot_width, transform_img_width, [crop_left, crop_right])
            self.si.main_part_offset(x * img.width + origin_slot_width / scale_x / 2, center * img.width, 'x')
        else:
            # 宽铺满的情况下,等比放缩高  裁剪高，往上移动裁剪框
            width_scale = origin_slot_width / img_width
            scale_x = scale_y = width_scale
            x = 0
            y = 0
            self.si.main_part_offset(0.5, 0.5, 'y')

        self.si.img_scale_ratio(scale_x, scale_y)
        return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0, img_width, img_height

    def center_crop(self, origin_slot_width, origin_slot_height, img_width, img_height):
        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / img_height
        transform_img_width = img_width * height_scale
        if transform_img_width >= origin_slot_width:  # 图片宽度大于槽的宽度

            scale_x = scale_y = height_scale
            y = 0
            x = 0.5 * transform_img_width - origin_slot_width / 2
            self.si.main_part_offset(0.5, 0.5, 'x')
        else:
            # 宽铺满的情况下,等比放缩高  裁剪高
            width_scale = origin_slot_width / img_width
            scale_x = scale_y = width_scale
            transform_img_height = width_scale * img_height
            x = 0
            y = 0.5 * transform_img_height - origin_slot_height / 2
            self.si.main_part_offset(0.5, 0.5, 'y')

        self.si.img_scale_ratio(scale_x, scale_y)
        return x / scale_x, y / scale_x, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0, img_width, img_height

    def extension_crop(self, top, bottom, crop_point_result_dict, origin_slot, img, img_crop_type):
        # 一定要考虑图片放缩 后 相关点的变化

        origin_slot_width, origin_slot_height = origin_slot
        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / ((bottom - top) * img.height)
        transform_img_width = img.width * height_scale

        scale_x = scale_y = height_scale
        y = top
        # 加上padding的 左右点
        x1 = crop_point_result_dict['x1']
        x2 = crop_point_result_dict['x2']
        center = x1 + (x2 - x1) / 2

        left = center * transform_img_width - origin_slot_width / 2  # slot左边 距离 原点的位置
        right = center * transform_img_width + origin_slot_width / 2  # slot右边 距离 原点的位置

        tag_1 = (left >= 0)
        tag_2 = (right <= transform_img_width)

        if not (tag_1 and tag_2):
            self.si.extension()
        x = (center * transform_img_width - origin_slot_width / 2)
        # 调整 左右 扩边留白相等
        if tag_1 and tag_2:  # 左右 都有足够 预留范围 不用调整
            pass
        elif tag_1 and not tag_2:  # 左范围够 右范围不够 # 尝试 向 左移动
            if (right - transform_img_width) <= left:  # 右扩边 小于 左距离
                x = left - (right - transform_img_width)
            else:
                x = -((right - transform_img_width - left) / 2)
        elif not tag_1 and tag_2:  # 左范围不够 右范围够
            if -left <= (transform_img_width - right):
                x = 0
            else:
                x = (left + (transform_img_width - right)) / 2
        else:  # 左右范围都不够
            # 左右白边相加 求平均值
            x = -((right - transform_img_width) - left) / 2
        x = x / transform_img_width

        self.si.main_part_offset(x * img.width + origin_slot_width / scale_x / 2, center * img.width, 'x')

        self.si.img_scale_ratio(scale_x, scale_y)
        return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0

    def img_crop_detail_crop(self, top, bottom, left, right, origin_slot, img):
        """图片裁剪出 细节图"""

        origin_slot_width, origin_slot_height = origin_slot

        # 裁剪高铺满的情况下,等比放缩 裁剪 宽
        height_scale = origin_slot_height / ((bottom - top) * img.height)
        transform_img_width = img.width * height_scale

        if transform_img_width >= origin_slot_width:  # 图片宽度大于槽的宽度
            scale_x = scale_y = height_scale
            center = left + (right - left) / 2
            y = top
            x = self.alternative_center_crop(center, origin_slot_width, transform_img_width, [left, right])
            self.si.main_part_offset(0.5, 0.5, 'x')
        else:
            # 宽铺满的情况下,等比放缩 计算 高  裁剪高

            # 加上内边界后 裁剪点 x1,x2
            x1 = left
            x2 = right

            width_scale = origin_slot_width / ((x2 - x1) * img.width)
            # width_scale = origin_slot_width / img.width
            transform_img_height = width_scale * img.height
            scale_x = scale_y = width_scale
            x = x1  # 左位置为  加上内边界后 的左
            # x = 0  # 左位置为  加上内边界后 的左
            center = top + (bottom - top) / 2
            y = self.alternative_center_crop(center, origin_slot_height, transform_img_height, [top, bottom])
            self.si.main_part_offset(0.5, 0.5, 'y')
        self.si.img_scale_ratio(scale_x, scale_y)
        return x * img.width, y * img.height, origin_slot_width / scale_x, origin_slot_height / scale_x, scale_x, scale_y, 0


if __name__ == '__main__':
    slot_config = {
        "cr": {
            "n_c": 1,
            "m_t": {
                "ts": "human_top",
                "bs": "hip",
                "pU": "pi",
                "t": 20,
                "r": 0,
                "b": 30,
                "l": 0
            },
            "m_bu": {
                "ts": "human_top",
                "bs": "hip",
                "pU": "pi",
                "t": 20,
                "r": 0,
                "b": 30,
                "l": 0
            },
            "m_bo": {
                "ts": "garment_top",
                "bs": "human_bottom",
                "pU": "pi",
                "t": 0,
                "r": 0,
                "b": 0,
                "l": 0
            },
            "m_f": {
                "ts": "human_top",
                "bs": "hip",
                "pU": "pi",
                "t": 20,
                "r": 0,
                "b": 30,
                "l": 0
            },
            "t": {
                "ts": "garment_top",
                "bs": "garment_bottom",
                "pU": "pi",
                "t": 30,
                "r": 0,
                "b": 30,
                "l": 0
            }
        },
        # "s": "fix",
        "s": "e",
        "v": "2020-12-12",
        "width": 1280,
        "height": 1280,
        # "width": 750,
        # "height": 1000,
    }

    img_dict = {'id': '684245', 'filename': '2020-12-27 谜底LOOKBOOK82970.jpg',
                'storage_key': 'asset/1000189/dc2a1421-1c67-407a-b910-3e8088c55cdf.jpg',
                'image_classification': 'model-all_side', 'width': 3121, 'height': 4681, 'file_id': '723519',
                'recognition': '{"image": {"width": 533, "height": 800}, "image_category": "模特", "confidence": 0.9999964237213135, "human_bodies": [{"bbox": [115, 94, 371, 750], "confidence": 0.9444527626037598, "keypoints": [[205, 708, 0.3948015868663788], [151, 504, 0.32096853852272034], [169, 360, 0.5516449213027954], [231, 354, 0.5299614071846008], [231, 522, 0.25429943203926086], [295, 673, 0.36741459369659424], [197, 339, 0.406144917011261], [218, 258, 0.5439403653144836], [226, 188, 0.7137809991836548], [218, 125, 0.5797253251075745], [204, 309, 0.3430940508842468], [150, 316, 0.3954583406448364], [154, 208, 0.7125232219696045], [293, 210, 0.7758114337921143], [313, 299, 0.6915827393531799], [346, 390, 0.6816549301147461]], "keypoints_coco": [[370, 209, 0.19185090065002441], [-1, -1, 0.07802201807498932], [370, 191, 0.4209941327571869], [-1, -1, 0.015631435438990593], [370, 215, 0.5712090730667114], [-1, -1, 0.0031422432512044907], [370, 362, 0.4440481960773468], [-1, -1, 0.0031390266958624125], [263, 322, 0.9416176080703735], [-1, -1, 0.003667104057967663], [370, 239, 0.5986910462379456], [-1, -1, 0.04587884619832039], [370, 694, 0.7474750876426697], [-1, -1, 0.01830916479229927], [-1, -1, 0.0008104267180897295], [-1, -1, 0.013747957535088062], [-1, -1, 0.013147641904652119]], "orientation": 150, "face": {"bbox": [198, 117, 242, 178], "confidence": 0.9833429455757141, "landmarks": [[205, 140], [225, 134], [216, 150], [213, 164], [228, 160]]}}], "clothes": [{"bbox": [146, 691, 239, 754], "category_id": 6, "category": "shoes", "confidence": 0.8420706391334534, "keypoints": null}, {"bbox": [278, 665, 344, 750], "category_id": 6, "category": "shoes", "confidence": 0.8294224739074707, "keypoints": null}, {"bbox": [321, 448, 403, 598], "category_id": 7, "category": "bag", "confidence": 0.6695075631141663, "keypoints": null}, {"bbox": [103, 324, 302, 619], "category_id": 2, "category": "skirt", "confidence": 0.633188009262085, "keypoints": null}, {"bbox": [110, 131, 379, 434], "category_id": 1, "category": "blouse", "confidence": 0.5361945033073425, "keypoints": null}]}',
                'image_analysis': {
                    'human_pose': '{"image": {"width": 533, "height": 800}, "result": [{"bbox": [115, 94, 371, 750], "confidence": 0.9444527626037598, "keypoints": [[216, 150, 0.9833429455757141], [205, 140, 0.9833429455757141], [225, 134, 0.9833429455757141], [205, 140, 0.9833429455757141], [225, 134, 0.9833429455757141], [293, 210, 0.7758114337921143], [154, 208, 0.7125232219696045], [313, 299, 0.6915827393531799], [150, 316, 0.3954583406448364], [346, 390, 0.6816549301147461], [204, 309, 0.3430940508842468], [231, 354, 0.5299614071846008], [169, 360, 0.5516449213027954], [231, 522, 0.25429943203926086], [151, 504, 0.32096853852272034], [295, 673, 0.36741459369659424], [205, 708, 0.3948015868663788]]}]}',
                    'cloth_1_classification': '{"image": {"width": 533, "height": 800}, "result": [{"cat_id": 3, "cat_name": "模特（侧面）", "confidence": 0.9999964237213135}]}',
                    'cloth_1_info': '{"image": {"width": 533, "height": 800}, "result": [{"bbox": [146, 691, 239, 754], "cat_id": 6, "cat_name": "shoes", "confidence": 0.8420706391334534}, {"bbox": [278, 665, 344, 750], "cat_id": 6, "cat_name": "shoes", "confidence": 0.8294224739074707}, {"bbox": [321, 448, 403, 598], "cat_id": 7, "cat_name": "bag", "confidence": 0.6695075631141663}, {"bbox": [103, 324, 302, 619], "cat_id": 2, "cat_name": "skirt", "confidence": 0.633188009262085}, {"bbox": [110, 131, 379, 434], "cat_id": 1, "cat_name": "blouse", "confidence": 0.5361945033073425}]}',
                    'storage_key': 'asset/1000189/dc2a1421-1c67-407a-b910-3e8088c55cdf.jpg',
                    'recognition': '123123',
                }}

    # print('-------------开始裁剪--------------')
    image_crop = ImageCrop()
    result = image_crop.crop(slot_config, img_dict)
    # print('-------------结束裁剪--------------')

    # print('评价指标：', image_crop.si.crop_indexing_dict)
    # print('裁剪结果：', result)



# !/usr/bin/env python
# -*- coding:utf-8 -*-
# @author :libo

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="crop_utils",
    version="0.0.14",
    author="libo",
    author_email="6878595@qq.com",
    description="image crop utils 资源图裁剪打分",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://git.chaomy.com/libo/ecpro-utils.git",
    project_urls={
        "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)


import json

IMG_CLASSIFY = [['模特图', '模特上身图', '模特上身正面'], ['模特图', '模特上身图', '模特上身侧面'], ['模特图', '模特上身图', '模特上身背面'],
                ['模特图', '模特胸像图', '模特胸像正面'], ['模特图', '模特胸像图', '模特胸像侧面'], ['模特图', '模特胸像图', '模特胸像背面'],
                ['模特图', '模特下身图', '模特下身正面'], ['模特图', '模特下身图', '模特下身侧面'], ['模特图', '模特下身图', '模特下身背面'],
                ['模特图', '模特全身图', '模特全身正面'], ['模特图', '模特全身图', '模特全身侧面'], ['模特图', '模特全身图', '模特全身背面'],
                ['细节图', '面料'], ['细节图', '领部'], ['细节图', '肩部'], ['细节图', '腰部'], ['细节图', '衣袖'], ['细节图', '下摆'],
                ['细节图', '口袋'], ['细节图', '门襟'], ['细节图', '内衬'], ['细节图', '图案'], ['细节图', '裤脚'],
                ['平铺图', '平铺图背面'], ['平铺图', '平铺图正面'],
                ['资质图', '吊牌图'], ['资质图', '洗水唛'],
                ['动图']
                ]

CROP_TYPE = {
    'm_t': ['模特上身正面', '模特上身侧面', '模特上身背面'],
    'm_bu': ['模特胸像正面', '模特胸像侧面', '模特胸像背面'],
    'm_bo': ['模特下身正面', '模特下身侧面', '模特下身背面'],
    'm_f': ['模特全身正面', '模特全身侧面', '模特全身背面'],
    't': ['平铺图背面', '平铺图正面'],
    'b': ['资质图', '吊牌图', '洗水唛'],
    'g': ['动图'],
}


class Image:
    def __init__(self, data):
        #   1. 图片分类
        #   2. 颜色
        #   3. 裁剪范围
        #   4. 图片中人的数量，衣服的数量

        self.img_id = data.get('id')

        self.storage_key = data.get('storage_key')

        self.img_format = data.get('format')
        self.width = data.get('width')
        self.height = data.get('height')

        try:
            self.ratio = self.width / self.height
        except TypeError:
            raise TypeError(f"图片错误，请前往图片包页面将错误图片删除后再重试")
        except ZeroDivisionError:
            raise ZeroDivisionError(f"图片错误，请前往图片包页面将错误图片删除后再重试")
        except Exception:
            # logging.error("图片宽高信息错误", exc_info=True)
            raise Exception(f"图片错误，请前往图片包页面将错误图片删除后再重试")

        self.hot_area = None  # 自动生成热区
        self.hot_area_type = None  # 热区类型 url(链接) goods(货号)
        self.hand_hot_area_data(data.get('hot_area', {}))

        self.copy_writing = data['copy_writing'] if data.get('copy_writing') else ''
        self.name_slot = data.get('name_slot')
        self.name_slot_index = data.get('name_slot_index', 0)

        self.color = data.get('color', 1)  # 根据命名的颜色文件夹来划分 颜色 1 2 3 ·····
        self.color_name = data.get("color_name")  # 根据命名的颜色文件夹来划分 颜色 1 2 3 ·····
        self.filename = data.get("filename")  # 根据命名的颜色文件夹来划分 颜色 1 2 3 ·····

        image_analysis = data.get('image_analysis')
        if not image_analysis:
            image_analysis = {}

        self.ai_image_classification = '未识别'

        self.image_classification = self.amend_image_classification(data.get('image_classification'))  # 图片分类

        for i in IMG_CLASSIFY:
            if self.image_classification == i[-1]:
                self.image_classification_list = i
                break
        else:
            self.image_classification_list = [self.image_classification]  # 所属图片分类列表

        self.img_crop_type = None
        for t, img_class_list in CROP_TYPE.items():
            if self.image_classification in img_class_list:
                self.img_crop_type = t

        image_analysis['storage_key'] = self.storage_key
        self.humans = Human(image_analysis)
        self.cloths = Cloths(image_analysis)

        self.image_analysis = image_analysis
        # 获取主商品
        self.crop_point = self.deal_crop_key_point()

        self.image_detail = ImageDetail()
        self.detail_point = {}  # {'裁剪模特帽子':{},}

        if self.img_crop_type == 'b':
            self.brand = Brand(image_analysis)
            self.brand_crop_point = self.brand.deal_brand_crop_point()

        self.match_crop_detail = []

    def hand_hot_area_data(self, area_data):
        """
        自己根据 area content 添加 hot_area_type
        """
        if not area_data:
            return

        def is_url(hot_area):
            """判断热区是否为url"""
            return (hot_area and
                    (hot_area.get("hot_tb") or hot_area.get('hot_jd') or hot_area.get('hot_tm') or hot_area.get(
                        'hot_yz')))

        image_hot_area = area_data.get("content")
        if image_hot_area:
            hot_area = json.loads(image_hot_area)
            if is_url(hot_area):
                self.hot_area_type = "url"
            elif hot_area is not None and hot_area.get("code", None) is not None:
                self.hot_area_type = "goods"
            # Frontend need an object, loads string data into json object
            self.hot_area = hot_area

    def amend_image_classification(self, image_classification):
        '''ImageClassificationCategoryList = [
                    {
                        'name':
                            '模特上身图',
                        'value':
                            'model-top',
                        'sub_category': [
                            {
                                'name': '上身图正面',
                                'sub_category': None,
                                'value': 'model-top_positive',
                                'default': True,
                            },
                            {
                                'name': '上身图侧面',
                                'sub_category': None,
                                'value': 'model-top_side',
                            },
                            {
                                'name': '上身图背面',
                                'sub_category': None,
                                'value': 'model-top_back',
                            },
                        ]
                    },
                    {
                        'name':
                            '模特胸像图',
                        'value':
                            'model-up',
                        'sub_category': [
                            {
                                'name': '胸像图正面',
                                'sub_category': None,
                                'value': 'model-up_positive',
                                'default': True,
                            },
                            {
                                'name': '胸像图侧面',
                                'sub_category': None,
                                'value': 'model-up_side',
                            },
                            {
                                'name': '胸像图背面',
                                'sub_category': None,
                                'value': 'model-up_back',
                            },
                        ]
                    },
                    {
                        'name':
                            '模特下身图',
                        'value':
                            'model-down',
                        'sub_category': [
                            {
                                'name': '下身图正面',
                                'sub_category': None,
                                'value': 'model-down_positive',
                                'default': True,
                            },
                            {
                                'name': '下身图侧面',
                                'sub_category': None,
                                'value': 'model-down_side',
                            },
                            {
                                'name': '下身图背面',
                                'sub_category': None,
                                'value': 'model-down_back',
                            },
                        ]
                    },
                    {
                        'name':
                            '模特全身图',
                        'value':
                            'model-all',
                        'sub_category': [
                            {
                                'name': '全身图正面',
                                'sub_category': None,
                                'value': 'model-all_positive',
                                'default': True,
                            },
                            {
                                'name': '全身图侧面',
                                'sub_category': None,
                                'value': 'model-all_side',
                            },
                            {
                                'name': '全身图背面',
                                'sub_category': None,
                                'value': 'model-all_back',
                            },
                        ]
                    },
                    {
                        'name':
                            '平铺图',
                        'sub_category': [
                            {
                                'name': '正面',
                                'sub_category': None,
                                'value': 'tile_positive',
                                'default': True,
                            },
                            {
                                'name': '背面',
                                'sub_category': None,
                                'value': 'tile_back',
                            },
                        ],
                        'value':
                            'tile',
                    },
                    {
                        'name':
                            '细节图',
                        'sub_category': [
                            {
                                'name': '面料图',
                                'sub_category': None,
                                'value': 'detail_fabric',
                            },
                            {
                                'name': '其他',
                                'sub_category': None,
                                'value': 'detail_other',
                                'default': True,
                            },
                        ],
                        'value':
                            'detail',
                    },
                    {
                        'name':
                            '资质图',
                        'sub_category': [
                            {
                                'name': '吊牌',
                                'sub_category': None,
                                'value': 'intelligence_drop',
                                'default': True,
                            },
                            {
                                'name': '耐久性标签',
                                'sub_category': None,
                                'value': 'intelligence_tag',
                            },
                        ],
                        'value':
                            'intelligence',
                    },
                    {
                        'name':
                            '其他分类',
                        'sub_category': [
                            {
                                'name': '其他',
                                'sub_category': None,
                                'value': 'other',
                                'default': True,
                            },
                        ],
                        'value':
                            'other-category',
                    },
                ]'''
        cat_mapping = {'model': '模特', 'top': '上身', 'up': "胸像", "down": "下身", "all": "全身",
                       'tile': "平铺图",
                       "detail": "细节图", "fabric": "面料", "other": "其他",
                       "intelligence": "", 'drop': "吊牌图", 'tag': "洗水唛",
                       "positive": "正面", "side": "侧面", "back": "背面",
                       "sweep": "下摆", "lining": "内衬", "pocket": "口袋",
                       "pattern": "图案", "shoulder": "肩部", "waist": "腰部",
                       "sleeve": "衣袖", "leg": "裤脚", "stay": "门襟",
                       "collar": "领部",
                       "gif": "动图"
                       }
        try:
            # 素材中心，分类不存在 或 不存在识别分类，则按照ai识别结果的分类处理
            if not image_classification or ('no-watch' in image_classification):
                return '未识别'
            # 其他分类
            if '_' not in image_classification:
                return cat_mapping[image_classification]
            cat, sub_category = image_classification.split('_')
            if '-' not in cat:
                if sub_category != "other":
                    if cat == 'tile':
                        return cat_mapping[cat] + cat_mapping[sub_category]
                    return cat_mapping[sub_category]
                if cat == 'detail':
                    # 来个默认细节图
                    return "细节图"
                # 保持原有识别
                return self.ai_image_classification
            model, body = cat.split('-')
            return cat_mapping[model] + cat_mapping[body] + cat_mapping[sub_category]
        except Exception as e:
            # logging.error(f"图片{self.filename}, 分类解析失败: {e}", exc_info=True)
            return '未识别'

    def deal_crop_key_point(self, cloth_classify=(1, 2, 3, 4, 5), main_product_crop=None):
        '''
        cloth_classify 当前详情页 图片分类是 上身 下身 全身

        >=1 人时如何裁剪 取最大值 或最小值
        每个裁剪点 均为： 矩形区域 top_site:left,top   bottom_site:right,bottom

        图片顶部：pic_top
        图片底部：pic_bottom

        头顶：human_top
        脚底：human_bottom

        衣服上边:garment_top
        衣服下边:garment_bottom

        眼睛：eye
        鼻子：nose
        耳朵：ear
        肩膀：shoulder
        肘部：elbow
        手腕：wrist
        臀部：hip
        膝盖：knee
        脚踝：ankle
        :return:
        '''

        if main_product_crop and self.image_analysis.get('recognition'):  # 新版本ai识别结果,存在主商品
            recognition = json.loads(self.image_analysis["recognition"])
            main_product_category_id = recognition.get("main_product", {}).get("category_id")
            if main_product_category_id:
                cloth_classify = [main_product_category_id]

        crop_point_dict = dict()
        crop_point_dict['pic_top'] = crop_point_dict['pic_bottom'] = {
            'top_site': [0, 0],
            'bottom_site': [1, 1]
        }
        crop_point_dict = self.humans.deal_human_crop_point(crop_point_dict)

        self.cloths.cloth_classify = cloth_classify  # 期望的衣服
        crop_point_dict = self.cloths.deal_cloth_crop_point(crop_point_dict)
        self.crop_point = crop_point_dict
        return crop_point_dict

    def deal_crop_detail_img(self, ai_result):
        """传入 ai 识别结果，获取到 各个裁剪细节"""

        crop_name = '裁剪'
        if '模特图' in self.image_classification_list:
            crop_name += '模特'
        elif '平铺图' in self.image_classification_list:
            crop_name += '平铺'
        elif '细节图' in self.image_classification_list:
            crop_name += '细节'
        detail = self.image_detail.seek_detail_img(ai_result)

        crop_point_mapping = {
            'shoulder': '肩部',
            'cuff': '袖口',
            'waistline': '腰部',
            'hemline': '下摆',
            'bottom': '裤脚',
            'hat': '帽子',
            'pocket': '口袋',
        }
        # 该图片存在哪些 裁剪细节点
        for k, v in detail.items():
            crop_detail = crop_point_mapping.get(k)
            if crop_detail:
                self.detail_point[crop_name + crop_detail] = v


class Human:
    '''人的ai数据处理'''
    # 17 个 人体关键点对应关系 旧版ai识别逻辑
    HUMAN_BODY_KEY_POINTS = [
        # face
        "nose",
        "left_eye", "right_eye",
        "left_ear", "right_ear",
        # upper body
        "left_shoulder", "right_shoulder",
        "left_elbow", "right_elbow",
        "left_wrist", "right_wrist",
        # middle body
        "left_hip", "right_hip",
        # lower body
        "left_knee", "right_knee",
        "left_ankle", "right_ankle"
    ]
    # 裁剪点 映射 ai 识别的 人体关键点  旧版ai识别逻辑
    mapping = {
        'eye': ["left_eye", "right_eye"],
        "nose": ["nose", "nose"],
        "ear": ["left_ear", "right_ear"],
        "shoulder": ["left_shoulder", "right_shoulder"],
        "elbow": ["left_elbow", "right_elbow"],
        "wrist": ["left_wrist", "right_wrist"],
        "hip": ["left_hip", "right_hip"],
        "knee": ["left_knee", "right_knee"],
        "ankle": ["left_ankle", "right_ankle"],
    }

    def __init__(self, image_analysis):
        self.threshold = 2.5  # 置信度 2.5
        self.image_analysis = image_analysis
        self.head = False  # 头是否存在，以此来判断 头顶是否存在
        if image_analysis.get('recognition'):  # 新版本ai识别结果
            self.threshold = 0.05
            image_analysis['human_pose'] = self.add_new_crop_point(image_analysis.get("recognition"),
                                                                   image_analysis.get("human_pose"))
        self.image_width = None
        self.image_height = None
        self.human_body_width = None
        self.human_bodies_list = self.ai_human_pose(image_analysis.get('human_pose'))


    def calculate_mouth(self, ai_crop_point_dict):
        """
        1.根据 眼镜 鼻子 计算嘴巴的位置
        2.根据眼睛鼻子判断 头顶是否要删除
        3.根据脚踝判断 脚底是否要删除
        """
        # y: 鼻子的y + (鼻子的y - 眼睛最低的y)
        # x: 鼻子的x
        if 'eye' in ai_crop_point_dict.keys() and 'nose' in ai_crop_point_dict.keys():
            eye_y = min(ai_crop_point_dict['eye']['top_site'][1], ai_crop_point_dict['eye']['bottom_site'][1])
            nose_y = ai_crop_point_dict['nose']['top_site'][1]
            mouth_y = nose_y + (nose_y - eye_y)
            nose_x = ai_crop_point_dict['nose']['top_site'][0]
            ai_crop_point_dict['mouth'] = {
                'top_site': (nose_x, mouth_y),
                'bottom_site': (nose_x, mouth_y)
            }

        if 'eye' not in ai_crop_point_dict.keys() and 'nose' not in ai_crop_point_dict.keys() \
                and not self.head and 'human_top' in ai_crop_point_dict.keys():
            ai_crop_point_dict.pop('human_top')

        if 'ankle' not in ai_crop_point_dict.keys() and 'human_bottom' in ai_crop_point_dict.keys():
            ai_crop_point_dict.pop('human_bottom')

        return ai_crop_point_dict

    def calculate_mandible_crop_point(self, face, human_keypoint_list):
        """
        在recognition face 内
        置信度小于0.8不可信
        嘴巴点 在 keypoints index 3 4
        下巴点 在 bbox index  2 3

        face 存在 使用 下巴+嘴巴 计算下颚
        face 不存在 使用 人体关键点肩部和颈部 来计算下颚

        :return: 下颚 的点
        """

        mandible_point = [-1, -1, 0]

        # ['右踝', '右膝', '右臀', '左臀', '左膝', '左踝', '腰', '脊柱', '颈', '头', '右腕', '右肘', '右肩', '左肩', '左肘', '左腕']
        r_ank, r_kne, r_hip, l_hip, l_kne, l_ank, pel, spi, nec, hea, r_wri, r_elb, r_sho, l_sho, l_elb, l_wri = human_keypoint_list

        if hea[2] > 0:
            self.head = True

        # face 存在识别结果
        # 置信度小于0.8不可信
        if face and face.get('confidence', 0) >= 0.8:
            mandible_point[2] = face['confidence']
            # 下巴y点
            law_y = face['bbox'][3]  # 下巴纵轴
            left_mouth = face['landmarks'][3]  # 嘴的左边
            right_mouth = face['landmarks'][4]  # 嘴的右边
            # 嘴巴的中点
            mouth_x = min(left_mouth[0], right_mouth[0]) + abs(left_mouth[0] - right_mouth[0]) / 2
            mouth_y = max(left_mouth[1], right_mouth[1])
            # 下颚的点
            mandible_point[0] = mouth_x
            mandible_point[1] = min(law_y, mouth_y) + abs(mouth_y - law_y) / 2
            mandible_point[2] = 1
            # print(mandible_point)
        else:
            # print(r_sho, l_sho, pel, spi, nec)  # '右肩', '左肩' '腰', '脊柱', '颈'
            # 根据肩部和颈部计算下颚的点
            # 计算肩部点
            if r_sho[2] > 0.2 and l_sho[2] > 0.2:
                sho_y = min(r_sho[1], l_sho[1])
            elif r_sho[2] > 0.2:
                sho_y = r_sho[2]
            elif l_sho[2] > 0.2:
                sho_y = l_sho[2]
            else:
                # 无法判断肩膀位置，推出
                sho_y = None

            nec_y = None
            nec_x = None
            if nec[2] > 0.2:
                nec_y = nec[1]
                nec_x = nec[0]

            if nec_x is None or nec_y is None or sho_y is None:
                return mandible_point

            mandible_point[0] = nec_x
            mandible_point[1] = min(nec_y, sho_y) - abs(sho_y - nec_y) * 0.7
            mandible_point[2] = 1
        return mandible_point

    def calculate_x_human_center(self, crop_top, crop_bottom):

        """
        1.优先根据 左臀、右臀、腰、脊柱来计算，
        2.算不出来则 选择人体衣服外边框

        crop_top : 当前图片裁剪的上点
        crop_bottom： 当前图片裁剪的下点
        :return:
        """
        confidence = 0.4  # 人体主干关键点 置信度需要大于0.4

        if not self.image_analysis.get('recognition'):  # 新版本ai识别结果不存在
            return None

        recognition = json.loads(self.image_analysis.get('recognition'))  # 识别结果内不存在人体关键点
        human_bodies_list = recognition.get('human_bodies')
        if not human_bodies_list:
            return None

        all_human_center = []

        for i, human in enumerate(human_bodies_list):
            human_keypoint_list = human['keypoints']
            # ['右踝', '右膝', '右臀', '左臀', '左膝', '左踝', '腰', '脊柱', '颈', '头', '右腕', '右肘', '右肩', '左肩', '左肘', '左腕']
            r_ank, r_kne, r_hip, l_hip, l_kne, l_ank, pel, spi, nec, hea, r_wri, r_elb, r_sho, l_sho, l_elb, l_wri = human_keypoint_list

            # 有效的点
            valid_point_list = []

            # 不分左右的点 腰 pel、脊柱 spi
            # 1.置信度够
            # 2.在裁剪范围内
            pel_h = pel[1] / self.image_height
            pel_w = pel[0] / self.image_width
            if pel[2] >= confidence and crop_top <= pel_h <= crop_bottom:
                valid_point_list.append(pel_w)

            spi_h = spi[1] / self.image_height
            spi_w = spi[0] / self.image_width
            if spi[2] >= confidence and crop_top <= spi_h <= crop_bottom:
                valid_point_list.append(spi_w)

            # 分左右的点 左臀 l_hip、右臀 r_hip
            # 1. 左右点置信度都够
            # 2. 有一个点的 上下在裁剪范围内
            l_hip_h = l_hip[1] / self.image_height
            r_hip_h = r_hip[1] / self.image_height
            l_hip_w = l_hip[0] / self.image_width
            r_hip_w = r_hip[0] / self.image_width

            if l_hip[2] >= confidence and r_hip[2] >= confidence and \
                    (crop_top <= l_hip_h <= crop_bottom or crop_top <= r_hip_h <= crop_bottom):
                hip_w = l_hip_w + (r_hip_w - l_hip_w) / 2

                valid_point_list.append(hip_w)

            # 在有效点内计算人体中心点
            if valid_point_list:
                current_human_center = sum(valid_point_list) / len(valid_point_list)
                all_human_center.append(current_human_center)

        if not all_human_center:
            return None

        # 多人 取多人的中点
        return sum(all_human_center) / len(all_human_center)

    def add_new_crop_point(self, recognition, human_pose):
        """
        把 recognition 内的数据 添加到 旧版的17个人体关键点的数据结构中

        添加识别点
            腰部：waist
            下颚：mandible
        recognition 数据解释
        https://www.teambition.com/project/5e0dba42b2bba90021d9a872/app/5eba5fba6a92214d420a3219/workspaces/5fe40d962aa6d50046cc2bff/docs/60c1764beaa11900011f146e
        """
        if (human_pose is None) or (not human_pose):
            return human_pose
        human_pose_dict = json.loads(human_pose)
        human_pose_result = human_pose_dict["result"]

        recognition = json.loads(recognition)
        human_bodies_list = recognition.get('human_bodies')
        if (human_bodies_list is None) or (not human_bodies_list):
            return human_pose

        # 人的数量保持一致
        if len(human_bodies_list) != len(human_pose_result):
            return human_pose

        for i, human in enumerate(human_bodies_list):
            if len(human_pose_result[i]['keypoints']) == 17:
                # 倒着insert 不会改变前面点的index
                # 在recognition keypoints 内 index 为6 的是腰部
                human_pose_result[i]['keypoints'].insert(self.HUMAN_BODY_KEY_POINTS.index('left_hip'),
                                                         human['keypoints'][6])

                # 在recognition face 内
                # 置信度小于0.8不可信
                # 嘴巴点 在 keypoints index 3 4
                # 下巴点 在 bbox index  2 3
                human_pose_result[i]['keypoints'].insert(self.HUMAN_BODY_KEY_POINTS.index('left_shoulder'),
                                                         self.calculate_mandible_crop_point(human['face'],
                                                                                            human['keypoints']))

        # 添加完裁剪点后，下面两个配置需要对应的改掉
        # 19 个 人体关键点对应关系
        self.HUMAN_BODY_KEY_POINTS = [
            # face
            "nose",
            "left_eye",
            "right_eye",
            "left_ear",
            "right_ear",
            "mandible",
            # upper body
            "left_shoulder",
            "right_shoulder",
            "left_elbow",
            "right_elbow",
            "left_wrist",
            "right_wrist",
            # middle body
            "waist",
            "left_hip",
            "right_hip",
            # lower body
            "left_knee",
            "right_knee",
            "left_ankle",
            "right_ankle",
        ]
        # 裁剪点 映射 ai 识别的 人体关键点
        self.mapping = {
            "eye": ["left_eye", "right_eye"],
            "nose": ["nose", "nose"],
            "ear": ["left_ear", "right_ear"],
            "mandible": ["mandible", "mandible"],
            "shoulder": ["left_shoulder", "right_shoulder"],
            "elbow": ["left_elbow", "right_elbow"],
            "wrist": ["left_wrist", "right_wrist"],
            "waist": ["waist", "waist"],
            "hip": ["left_hip", "right_hip"],
            "knee": ["left_knee", "right_knee"],
            "ankle": ["left_ankle", "right_ankle"],
        }

        human_pose_dict['result'] = human_pose_result

        human_pose = json.dumps(human_pose_dict)

        # except Exception:
        #     logging.error(f"ai识别结果-人体关键点-解析失败: {self.image_analysis} ", exc_info=True)
        return human_pose

    def deal_human_crop_point(self, crop_point_dict):
        '''
        处理人体的裁剪点，方便裁剪时取值
        '''
        for human in self.human_bodies_list:
            crop_point_dict = self.human_crop_point(human, crop_point_dict)

        # 计算嘴巴的位置
        crop_point_dict = self.calculate_mouth(crop_point_dict)

        return crop_point_dict

    def human_crop_point(self, human, crop_point_dict):
        '''
        眼睛：eye  矩形区域 top_site:left,top   bottom_site:right,bottom
        鼻子：nose
        耳朵：ear
        肩膀：shoulder
        肘部：elbow
        手腕：wrist
        臀部：hip
        膝盖：knee
        脚踝：ankle

        头顶：human_top
        脚底：human_bottom

        :return:
        '''

        human_body = human['human_key_point']
        # 关键点
        for point_k, point_v_list in self.mapping.items():

            # point_v_list 为 左点 右点  例如： ["left_eye", "right_eye"]
            # 关键点可见置信度阈值设置为 2.5
            # 左点
            x1, y1 = human_body[point_v_list[0]][0], human_body[point_v_list[0]][1]
            # 右点
            x2, y2 = human_body[point_v_list[1]][0], human_body[point_v_list[1]][1]

            # 人体识别点 分左右点 合并 左右点为一个矩形
            # 左右点 均存在
            point = {}
            if human_body[point_v_list[0]][2] > self.threshold and human_body[point_v_list[1]][2] > self.threshold:
                point['top_site'] = x1, y1
                point['bottom_site'] = x2, y2
            # 左点存在
            elif human_body[point_v_list[0]][2] > self.threshold:
                point['top_site'] = point['bottom_site'] = x1, y1
            # 右点存在
            elif human_body[point_v_list[1]][2] > self.threshold:
                point['top_site'] = point['bottom_site'] = x2, y2
            else:
                continue

            # 可能存在多个人 即存在多个相同分类 的人体关键点
            # 若不存在裁剪点
            if not crop_point_dict.get(point_k):
                crop_point_dict[point_k] = point
                continue
            human_top_site = crop_point_dict[point_k]['top_site']
            human_bottom_site = crop_point_dict[point_k]['bottom_site']
            # 存在裁剪点
            # 取最小
            crop_point_dict[point_k]['top_site'] = min(human_top_site[0], point['top_site'][0]), \
                                                   min(human_top_site[1], point['top_site'][1])
            # 取最大
            crop_point_dict[point_k]['bottom_site'] = max(human_bottom_site[0], point['bottom_site'][0]), \
                                                      max(human_bottom_site[1], point['bottom_site'][1])

        # 人的 头顶 脚底
        human_box = human['box']
        if not crop_point_dict.get('human_top'):
            point = dict()
            point['top_site'] = human_box[0], human_box[1]
            point['bottom_site'] = human_box[2], human_box[3]
            crop_point_dict['human_top'] = crop_point_dict['human_bottom'] = point
            return crop_point_dict

        human_top_site = crop_point_dict['human_top']['top_site']
        human_bottom_site = crop_point_dict['human_top']['bottom_site']
        # 取最小
        crop_point_dict['human_top']['top_site'] = min(human_top_site[0], human_box[0]), min(human_top_site[1],
                                                                                             human_box[1])
        # 取最大
        crop_point_dict['human_top']['bottom_site'] = max(human_bottom_site[0], human_box[2]), max(human_bottom_site[1],
                                                                                                   human_box[3])
        crop_point_dict['human_bottom'] = crop_point_dict['human_top']
        return crop_point_dict

    def ai_human_pose(self, human_pose):
        human_bodies = []
        if not human_pose:
            return human_bodies
        try:
            human_pose = json.loads(human_pose)
            self.image_width, self.image_height = human_pose['image']['width'], human_pose['image']['height']

            for one_human_data in human_pose['result']:
                human_body = self.deal_human(one_human_data)
                human_bodies.append(human_body)
        except Exception:
            raise Exception(f"ai识别结果-人体关键点-解析失败: {self.image_analysis} ")
            # logging.error(f"ai识别结果-人体关键点-解析失败: {self.image_analysis} ", exc_info=True)
        return human_bodies

    def deal_human(self, one_human_data):
        '''key: 各个人体关键点 value: x坐标，y坐标，置信度'''
        human_body = dict()  # box ; human_key_point
        human_body['box'] = [
            one_human_data['bbox'][0] / self.image_width,
            one_human_data['bbox'][1] / self.image_height,
            one_human_data['bbox'][2] / self.image_width,
            one_human_data['bbox'][3] / self.image_height
        ]

        if not len(self.HUMAN_BODY_KEY_POINTS) == len(one_human_data['keypoints']):
            return human_body

        key_points_list = one_human_data['keypoints']
        human_key_point = dict()
        for key_point_index, key_point in enumerate(self.HUMAN_BODY_KEY_POINTS):
            _key_point = [key_points_list[key_point_index][0] / self.image_width,
                          key_points_list[key_point_index][1] / self.image_height,
                          key_points_list[key_point_index][2]
                          ]
            human_key_point[key_point] = _key_point
        human_body['human_key_point'] = human_key_point
        return human_body


class Cloths:
    '''衣服的ai数据处理'''

    def __init__(self, image_analysis):
        self.image_analysis = image_analysis
        self.cloths_list = self.ai_cloth_1_info(image_analysis.get('cloth_1_info'))
        self.image_width = None
        self.image_height = None
        self.cloth_classify = None
        self.cloth_width = None
        self.cloth_height = None

    def deal_cloth_crop_point(self, crop_point_dict):

        for cloth in self.cloths_list:
            crop_point_dict = self.cloth_crop_point(cloth, crop_point_dict)
        return crop_point_dict

    def cloth_crop_point(self, cloth, crop_point_dict):
        #   衣服上边:garment_top
        #   衣服下边:garment_bottom
        #   衣服的范围

        # 该件衣服的分类 不在所需的范围
        if cloth['cat_id'] not in self.cloth_classify:
            return crop_point_dict

        cloth_box = cloth['box']
        if not crop_point_dict.get('garment_top'):
            point = dict()
            point['top_site'] = cloth_box[0], cloth_box[1]
            point['bottom_site'] = cloth_box[2], cloth_box[3]
            crop_point_dict['garment_top'] = crop_point_dict['garment_bottom'] = point
            return crop_point_dict

        garment_top_site = crop_point_dict['garment_top']['top_site']
        garment_bottom_site = crop_point_dict['garment_bottom']['bottom_site']
        # 取最小
        crop_point_dict['garment_top']['top_site'] = min(garment_top_site[0], cloth_box[0]), min(garment_top_site[1],
                                                                                                 cloth_box[1])
        # 取最大
        crop_point_dict['garment_top']['bottom_site'] = max(garment_bottom_site[0], cloth_box[2]), max(
            garment_bottom_site[1], cloth_box[3])
        crop_point_dict['garment_bottom'] = crop_point_dict['garment_top']
        return crop_point_dict

    def ai_cloth_1_info(self, cloth_1_info):
        '''计算各件衣服的分类 及box '''
        if not cloth_1_info:
            return []
        cloths_list = []
        try:
            cloth_1_info = json.loads(cloth_1_info)
            self.image_width, self.image_height = cloth_1_info['image']['width'], cloth_1_info['image']['height']
            for one_garment in cloth_1_info['result']:
                garment = self.deal_garment(one_garment)
                cloths_list.append(garment)
        except Exception:
            raise Exception(f"ai识别结果-衣服box-解析失败: {json.dumps(self.image_analysis)}")
            # logging.error(f"ai识别结果-衣服box-解析失败: {json.dumps(self.image_analysis)} ", exc_info=True)
        return cloths_list

    def deal_garment(self, one_garment):
        '''处理下 每件衣服的 box cat_id cat_name confidence'''
        garment = dict()

        #  服装的区域
        garment['box'] = [
            one_garment['bbox'][0] / self.image_width,
            one_garment['bbox'][1] / self.image_height,
            one_garment['bbox'][2] / self.image_width,
            one_garment['bbox'][3] / self.image_height
        ]

        garment['cat_id'] = one_garment['cat_id']  # 服装分类id
        garment['cat_name'] = one_garment['cat_name']  # 服装分类name
        return garment


class Brand:
    def __init__(self, image_analysis):
        self.image_analysis = image_analysis
        self.threshold = 0.6  # 小于0.6不可信
        self.brand_list = self.ai_brand_info(image_analysis.get("recognition"))

    def ai_brand_info(self, brand_json):
        '''计算各个资质图的分类 及box '''

        if not brand_json:
            return []

        brand_list = []
        try:
            brand_info = json.loads(brand_json)
            if not brand_info.get('labels'):
                return []
            self.image_width, self.image_height = brand_info['image']['width'], brand_info['image']['height']
            for one_brand in brand_info['labels']:
                if one_brand['confidence'] < self.threshold:  # 置信度过低
                    continue
                brand = self.deal_brand(one_brand)
                brand_list.append(brand)
        except Exception:
            raise Exception(f"ai识别结果-资质图box-解析失败: {json.dumps(self.image_analysis)}")
            # logging.error(f"ai识别结果-资质图box-解析失败: {json.dumps(self.image_analysis)} ", exc_info=True)
        return brand_list

    def deal_brand(self, one_brand):
        '''处理下 每个资质图的 box cat_id cat_name confidence'''

        brand = dict()

        #  服装的区域
        brand['box'] = [
            one_brand['bbox'][0] / self.image_width,
            one_brand['bbox'][1] / self.image_height,
            one_brand['bbox'][2] / self.image_width,
            one_brand['bbox'][3] / self.image_height
        ]

        brand['cat_id'] = one_brand['category_id']  # 服装分类id
        brand['cat_name'] = one_brand['category']  # 资质图分类name
        return brand

    def deal_brand_crop_point(self):
        if not self.brand_list:
            return {
                'brand_top': {
                    'top_site': [0, 0], 'bottom_site': [1, 1]
                },
                'brand_bottom': {
                    'top_site': [0, 0], 'bottom_site': [1, 1]
                }}

        brand_crop_point_dict = {}
        for brand in self.brand_list:
            brand_crop_point_dict = self.brand_crop_point(brand, brand_crop_point_dict)
        return brand_crop_point_dict

    def brand_crop_point(self, brand, crop_point_dict):
        # 资质图上边:brand_top
        # 资质图下边:brand_bottom

        brand_box = brand['box']
        if not crop_point_dict.get('brand_top'):
            point = dict()
            point['top_site'] = brand_box[0], brand_box[1]
            point['bottom_site'] = brand_box[2], brand_box[3]
            crop_point_dict['brand_top'] = crop_point_dict['brand_bottom'] = point
            return crop_point_dict

        brand_top_site = crop_point_dict['brand_top']['top_site']
        brand_bottom_site = crop_point_dict['brand_bottom']['bottom_site']
        # 取最小
        crop_point_dict['brand_top']['top_site'] = min(brand_top_site[0], brand_box[0]), min(brand_top_site[1],
                                                                                             brand_box[1])
        # 取最大
        crop_point_dict['brand_top']['bottom_site'] = max(brand_bottom_site[0], brand_box[2]), max(
            brand_bottom_site[1], brand_box[3])
        crop_point_dict['brand_bottom'] = crop_point_dict['brand_top']
        return crop_point_dict


class ImageDetail:
    """
    TB链接：https://thoughts.teambition.com/workspaces/5e37b6adf0f331001a9c172d/docs/607e874e4cc5830001d956c7

    从图片中，根据ai识别结果，标识出出细节图的位置
    帽子 hat
    袖口
    口袋 pocket
    腰部
    下摆
    裤脚
    鞋子 shoes
    包   bag
    """
    threshold = 0.1  # 置信度 0.1 看看效果
    # 衣服分类 及分类下关键点映射关系
    cloth_classify = {
        'blouse': [
            'neckline_left',
            'neckline_right',
            'center_front',
            'shoulder_left',
            'shoulder_right',
            'armpit_left',
            'armpit_right',
            'cuff_left_in',
            'cuff_left_out',
            'cuff_right_out',
            'top_hem_left',
            'top_hem_right'
        ],
        'skirt': [
            'waistband_left',
            'waistband_right',
            'hemline_left',
            'hemline_right'
        ],
        'outwear': [
            'neckline_left',
            'neckline_right',
            'center_front',
            'shoulder_left',
            'shoulder_right',
            'armpit_left',
            'armpit_right',
            'waistline_left',
            'waistline_right',
            'cuff_left_in',
            'cuff_left_out',
            'cuff_right_in',
            'cuff_right_out',
            'top_hem_left',
            'top_hem_right',
        ],
        'dress': [
            'neckline_left',
            'neckline_right',
            'center_front',
            'shoulder_left',
            'shoulder_right',
            'armpit_left',
            'armpit_right',
            'waistline_left',
            'waistline_right',
            'cuff_left_in',
            'cuff_left_out',
            'cuff_right_in',
            'cuff_right_out',
            'hemline_left',
            'hemline_right'
        ],

        'trousers': [
            'waistband_left',
            'waistband_right',
            'crotch',
            'bottom_left_in',
            'bottom_left_out',
            'bottom_right_in',
            'bottom_right_out',
        ]
    }
    # 关键点 3.0.7版本
    cloth_key_point = {
        'shoulder': ['shoulder_left',
                     'shoulder_right'],
        'cuff': ['cuff_left_out',
                 'cuff_left_in',
                 'cuff_right_in',
                 'cuff_right_out'],
        'waistline': ['waistline_left',
                      'waistline_right',
                      'waistband_left',
                      'waistband_right'
                      ],
        'hemline': ['hemline_left',
                    'hemline_right',
                    'top_hem_left',
                    'top_hem_right'
                    ],
        'bottom': ['bottom_left_out',
                   'bottom_left_in',
                   'bottom_right_in',
                   'bottom_right_out']
    }
    # 目标检测 3.0.7版本
    # cloth_object_detection = ['shoes', 'bag', 'hat', 'pocket']
    cloth_object_detection = ['hat', 'pocket']

    def __init__(self):
        self.image_width = None
        self.image_height = None

        self.detail = {}
        self.match_crop_detail = []

    def seek_detail_img(self, ai_predictions):
        """寻找 细节图 的坐标信息"""
        # ai_predictions = {"predictions": [
        #     {
        #         "image": {
        #             "width": 800,
        #             "height": 533
        #         },
        #         "result": [
        #             {
        #                 "bbox": [
        #                     "311.0",
        #                     "361.0",
        #                     "504.0",
        #                     "533.0"
        #                 ],
        #                 "cat_id": 5,
        #                 "cat_name": "trousers",
        #                 "confidence": 0.7771915793418884,
        #                 "keypoints": [
        #                     [
        #                         63.4375,
        #                         30.8125,
        #                         0.29523932933807373
        #                     ],
        #                     [
        #                         177.625,
        #                         25.375,
        #                         0.24510465562343597
        #                     ],
        #                     [
        #                         106.9375,
        #                         148.625,
        #                         0.36413368582725525
        #                     ],
        #                     [
        #                         54.375,
        #                         68.875,
        #                         0.013722038827836514
        #                     ],
        #                     [
        #                         45.3125,
        #                         134.125,
        #                         0.21816827356815338
        #                     ],
        #                     [
        #                         50.75,
        #                         67.0625,
        #                         0.008496182970702648
        #                     ],
        #                     [
        #                         195.75,
        #                         188.5,
        #                         0.019060151651501656
        #                     ]
        #                 ]
        #             },
        #             {
        #                 "bbox": [
        #                     "319.0",
        #                     "136.0",
        #                     "544.0",
        #                     "419.0"
        #                 ],
        #                 "cat_id": 1,
        #                 "cat_name": "blouse",
        #                 "confidence": 0.5724542140960693,
        #                 "keypoints": [
        #                     [
        #                         100.9375,
        #                         45.15625,
        #                         0.6377096176147461
        #                     ],
        #                     [
        #                         156.71875,
        #                         42.5,
        #                         0.626152515411377
        #                     ],
        #                     [
        #                         127.5,
        #                         53.125,
        #                         0.7923415899276733
        #                     ],
        #                     [
        #                         61.09375,
        #                         69.0625,
        #                         0.890479326248169
        #                     ],
        #                     [
        #                         196.5625,
        #                         71.71875,
        #                         0.8325190544128418
        #                     ],
        #                     [
        #                         77.03125,
        #                         132.8125,
        #                         0.4232546389102936
        #                     ],
        #                     [
        #                         183.28125,
        #                         135.46875,
        #                         0.3931943476200104
        #                     ],
        #                     [
        #                         63.75,
        #                         225.78125,
        #                         0.439550518989563
        #                     ],
        #                     [
        #                         39.84375,
        #                         228.4375,
        #                         0.37197989225387573
        #                     ],
        #                     [
        #                         106.25,
        #                         228.4375,
        #                         0.8018227815628052
        #                     ],
        #                     [
        #                         132.8125,
        #                         268.28125,
        #                         0.6333439350128174
        #                     ],
        #                     [
        #                         42.5,
        #                         289.53125,
        #                         0.2018098682165146
        #                     ],
        #                     [
        #                         185.9375,
        #                         276.25,
        #                         0.18934710323810577
        #                     ]
        #                 ]
        #             },
        #             {
        #                 "bbox": [
        #                     "452.0",
        #                     "387.0",
        #                     "483.0",
        #                     "451.0"
        #                 ],
        #                 "cat_id": 9,
        #                 "cat_name": "pocket",
        #                 "confidence": 0.6082484126091003,
        #                 "keypoints": [
        #
        #                 ]
        #             }
        #         ]
        #     }]}['predictions'][0]
        self.image_width = ai_predictions['image']['width']
        self.image_height = ai_predictions['image']['height']

        ai_prediction_results = ai_predictions['result']
        for predict_part in ai_prediction_results:
            cat_name = predict_part['cat_name']

            # 衣服关键点
            if cat_name in self.cloth_classify.keys():
                self.deal_cloth_key_point(predict_part)
            # 目标检测
            elif cat_name in self.cloth_object_detection:
                self.deal_cloth_obj_detection(predict_part)
        return self.detail
        # print(self.detail)

    def add_crop_detail(self, img_classify):
        """根据图片类型添加 裁剪细节图"""
        if '平铺图' in img_classify:
            self.match_crop_detail.append('裁剪平铺细节图')
        elif '模特图' in img_classify:
            self.match_crop_detail.append('裁剪模特细节图')

    def deal_cloth_key_point(self, predict_part):
        """根据衣服的key_point结果，找到细节位置"""
        cloth_bbox = predict_part['bbox']
        predict_key_points = predict_part['keypoints']  # ai识别的keypoint 数据列表
        key_point_list = self.cloth_classify.get(predict_part['cat_name'], [])  # keypoint 代表具体部位的列表

        for cat_name, keypoint_part_list in self.cloth_key_point.items():

            for i in range(0, len(keypoint_part_list), 2):
                # 识别点 在 ai 识别结果的下标

                if (keypoint_part_list[i] not in key_point_list) or (keypoint_part_list[i + 1] not in key_point_list):
                    continue

                predict_index_left = key_point_list.index(keypoint_part_list[i])
                predict_index_right = key_point_list.index(keypoint_part_list[i + 1])

                top_site = predict_key_points[predict_index_left]
                bottom_site = predict_key_points[predict_index_right]
                # 置信度过低，不使用该位置

                # if top_site[2] < self.threshold or bottom_site[2] < self.threshold:
                #     continue

                confidence = (top_site[2] + bottom_site[2]) * 0.5

                if cat_name in self.detail.keys() and confidence < self.detail[cat_name]['confidence']:
                    continue

                self.detail[cat_name] = {
                    'top_site': ((top_site[0] + cloth_bbox[0]) / self.image_width,
                                 (top_site[1] + cloth_bbox[1]) / self.image_height),
                    'bottom_site': ((bottom_site[0] + cloth_bbox[0]) / self.image_width,
                                    (bottom_site[1] + cloth_bbox[1]) / self.image_height),
                    'confidence': confidence,
                    'top_site_confidence': top_site[2],
                    'bottom_site_confidence': bottom_site[2]
                }

    def deal_cloth_obj_detection(self, predict_part):
        """根据目标检测的结果，找到细节位置"""
        confidence = predict_part['confidence']
        # 置信度过低，不使用该位置
        # if confidence < self.threshold:
        #     return

        bounding_box = [float(i) for i in predict_part['bbox']]
        cat_name = predict_part['cat_name']

        # part已经存在且  置信度低于 上一个 part 时
        if cat_name in self.detail and confidence < self.detail[cat_name]['confidence']:
            return

        self.detail[cat_name] = {
            'top_site': (bounding_box[0] / self.image_width, bounding_box[1] / self.image_height),
            'bottom_site': (bounding_box[2] / self.image_width, bounding_box[3] / self.image_height),
            'confidence': confidence
        }


if __name__ == '__main__':
    a = ImageDetail()
